#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jan 30 15:07:27 2019

https://github.com/julienGautier77/visu.git

@author: juliengautier(LOA)
for dark style :
pip install qdarkstyle (https://github.com/ColinDuquesnoy/QDarkStyleSheet.git)

pip install pyqtgraph (https://github.com/pyqtgraph/pyqtgraph.git)
conda install pyopengl 3D plot

created 2021/11/02 : new design
"""



import pyqtgraph as pg  #pip install pyqtgraph (https://github.com/pyqtgraph/pyqtgraph.git)

from PyQt6.QtWidgets import QApplication, QVBoxLayout,QHBoxLayout,QPushButton
from PyQt6.QtWidgets import QInputDialog,QSlider,QLabel,QSizePolicy,QMenu,QMessageBox,QFileDialog
from PyQt6.QtWidgets import QMainWindow,QToolButton,QStatusBar,QFrame
from PyQt6.QtGui import QShortcut,QAction
from PyQt6 import QtCore,QtGui
from PyQt6.QtCore import pyqtSlot,Qt
from PyQt6.QtGui import QIcon

import sys,time,os

import numpy as np
import qdarkstyle # pip install qdarkstyle https://github.com/ColinDuquesnoy/QDarkStyleSheet  sur conda
from scipy.interpolate import splrep, sproot #
from scipy.ndimage import gaussian_filter,median_filter
from PIL import Image
from visu.winspec import SpeFile
from visu.visualLight import SEELIGHT
from visu.winSuppE import WINENCERCLED
from visu.WinCut import GRAPHCUT
from visu.winMeas import MEAS
from visu.WinOption import OPTION
from visu.WinPreference import PREFERENCES
from visu.andor import SifFile
from visu.winFFT import WINFFT
from visu.winMath import WINMATH
from visu.winPointing import WINPOINTING
from visu.winHist import HISTORY
from visu import aboutWindows
from visu.winZoom import ZOOM
# try :
#     from visu.Win3D import GRAPH3D #conda install pyopengl
# except :
#     print ('')
    
import pathlib
import visu

__version__=visu.__version__
__author__=visu.__author__


__all__=['SEE','runVisu']

class SEE(QMainWindow) :
    '''open and plot file : 
        SEE(file='nameFile,path=pathFileName,confpath,confMot,name,aff)
        Make plot profile ands differents measurements(max,min mean...)
        Can open .spe .SPE .sif .TIFF files
        
        
        file = name of the file to open
        path=path of the file to open
        confpath=path  and  file name of the ini file if =None ini file will be the one in visu folder
        confMot usefull if RSAI motors is read
        name= name of the item  in the ini file could be usfull if there is more than two visu widget open in the  same   time
            default is  VISU
        
        kwds :
            aff = "right" or "left" display button on the  right or on the left
            fft="on" or "off" display 1d and 2D fft 
            plot3D 
    '''
    
    signalMeas= QtCore.pyqtSignal(object)
    signalEng=QtCore.pyqtSignal(object)
    signalPointing=QtCore.pyqtSignal(object)
    signalPlot=QtCore.pyqtSignal(object)
    
    def __init__(self,file=None,path=None,parent=None,**kwds):
        
        super().__init__()
        version=__version__
        self.parent=parent
        print("data visualisation version :  ",version)
        p = pathlib.Path(__file__)
        self.fullscreen=False
        self.setAcceptDrops(True)
        sepa=os.sep
        self.icon=str(p.parent) + sepa+'icons' +sepa
        self.colorBar='flame'
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.nomFichier=''
        self.labelValue=''
        self.aboutWidget=aboutWindows.ABOUT()
        self.signalTrans=dict() #dict to emit multivariable
        self.frameNumber=0
        ### kwds definition  : 
        
        if "confpath"in kwds :   #confpath path.file pour le fichier ini.
            self.confpath=kwds["confpath"]
            if self.confpath==None:
                self.conf=QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
            else:
                self.conf=QtCore.QSettings(self.confpath, QtCore.QSettings.Format.IniFormat)
            print ('configuration path of visu : ',self.confpath)
            
            # print ('conf path visu',self.confpath,self.conf)
        else:
            self.conf=QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        
        if "conf"in kwds:               #conf : le QSetting
            self.conf=kwds["conf"]
        
        if "color" in kwds:
            self.color=kwds["color"]
        else:
            self.color=None
        
        if "name" in kwds:
            self.name=kwds["name"]
        else:
            self.name="VISU"
        
        print('name :' ,self.name)
        
        if "roiCross" in kwds:
            # print('ROICROSS on')
            self.roiCross=kwds["roiCross"]
        else :
            self.roiCross=False
            # print('no roicross')
        if "aff" in kwds:
            self.aff=kwds["aff"]
        else :
            self.aff="right"
        
        if "fft" in kwds:
            self.fft=kwds["fft"]
        else:
            self.fft="off"
        
        if "meas" in kwds:
            self.meas=kwds["meas"]
        else:
           self.meas="on" 
           
        if "encercled" in kwds:
            self.encercled=kwds["encercled"]
        else:
            self.encercled="on"
        
        
        if self.fft=='on'  :  
            self.winFFT=WINFFT(conf=self.conf,name=self.name)
            self.winFFT1D=GRAPHCUT(symbol=False,title='FFT 1D',conf=self.conf,name=self.name)
          
        if "filter" in kwds:
            self.winFilter=kwds['filter']
        else :
            self.winFilter='on'
            
        if "confMot" in kwds:
            print('motor accepted')
            if self.meas=="on":
                self.confMot=kwds["confMot"]
                # print(self.confMot)
                self.winM=MEAS(parent=self,confMot=self.confMot,conf=self.conf,name=self.name)
        else :
            if self.meas=="on":
                self.winM=MEAS(parent=self,conf=self.conf,name=self.name)
            
        self.winOpt=OPTION(conf=self.conf,name=self.name)
        self.winPref=PREFERENCES(conf=self.conf,name=self.name)
        self.winHistory=HISTORY(self,conf=self.conf,name=self.name)
        
        if self.encercled=="on":
            self.winEncercled=WINENCERCLED(parent=self,conf=self.conf,name=self.name)
        
        if "plot3d" in kwds :
            self.plot3D=kwds["plot3d"]
        else:
            self.plot3D="off"
            
        # if self.plot3D=="on":
            
        #     self.Widget3D=GRAPH3D(self.conf,name=self.name)
        
        
        if "math" in kwds:
            self.math=kwds["math"]
        else :
            self.math="on"
        
        if self.math=="on":
            self.winMath=WINMATH()
            
            
            
        self.winPointing=WINPOINTING(parent=self)
        self.winCoupe=GRAPHCUT(parent=self,symbol=None,conf=self.conf,name=self.name)
        self.path=path
        self.setWindowTitle('Visualization'+'       v.'+ version)
        self.bloqKeyboard=1#bool((self.conf.value(self.name+"/bloqKeyboard"))  )  # block cross by keyboard
        self.bloqq=1 # block the cross by click on mouse
        
        # initialize variable : 
        self.filter='origin' # filter initial value
        self.ite=None
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.zo=1 # zoom initial value
        self.scaleAxis="off"
        self.plotRectZoomEtat='Zoom'
        self.angleImage=0
        
        self.setup()
        
        def twoD_Gaussian(x,y, amplitude, xo, yo, sigma_x, sigma_y, theta, offset):
           xo = float(xo)
           yo = float(yo)    
           a = (np.cos(theta)**2)/(2*sigma_x**2) + (np.sin(theta)**2)/(2*sigma_y**2)
           b = -(np.sin(2*theta))/(4*sigma_x**2) + (np.sin(2*theta))/(4*sigma_y**2)
           c = (np.sin(theta)**2)/(2*sigma_x**2) + (np.cos(theta)**2)/(2*sigma_y**2)
           return offset + amplitude*np.exp( - (a*((x-xo)**2) + 2*b*(x-xo)*(y-yo) + c*((y-yo)**2)))

        
        if file==None:
            # to have a gaussian picture when we start
            self.dimy=800
            self.dimx=800
            # Create x and y index
            self.x = np.arange(0,self.dimx)
            self.y = np.arange(0,self.dimy)
            self.y,self.x = np.meshgrid(self.y, self.x)
            
            self.data=twoD_Gaussian(self.x, self.y,200, 200, 600, 40, 40, 0, 10)+(50*np.random.rand(self.dimx,self.dimy)).round() 
        
            #self.data=(50*np.random.rand(self.dimx,self.dimy)).round() + 150
        else:
            if path==None:
                self.path=self.conf.value(self.name+"/path")
            
            self.data=self.OpenF(fileOpen=self.path+'/'+file)
        
        
        self.dataOrg=self.data
        self.dataOrgScale=self.data
        
        self.xminR=0
        self.xmaxR=self.dimx
        self.yminR=0
        self.ymaxR=self.dimy
        
        
        self.shortcut()
        self.actionButton()
        self.Display(self.data)
        self.activateWindow()
        self.raise_()
        self.showNormal()
        
        
        
    def setup(self):
        # definition of all button 
        
        TogOff=self.icon+'Toggle_Off.png' 
        TogOn=self.icon+'Toggle_On.png'
        TogOff=pathlib.Path(TogOff)
        TogOff=pathlib.PurePosixPath(TogOff)
        TogOn=pathlib.Path(TogOn)
        TogOn=pathlib.PurePosixPath(TogOn)
        self.hboxBar=QHBoxLayout() 
        
        #self.setStyleSheet("QCheckBox::indicator{width: 30px;height: 30px;}""QCheckBox::indicator:unchecked { image : url(%s);}""QCheckBox::indicator:checked { image:  url(%s);}""QCheckBox{font :10pt;QCheckBox{background-color :red}" % (TogOff,TogOn) )
        
        
        self.toolBar =self.addToolBar('tools')
        self.toolBar.setMovable(False)
        #self.setStyleSheet("{background-color: black}")
        menubar = self.menuBar()
        menubar.setNativeMenuBar(False)
        self.fileMenu = menubar.addMenu('&File')
        self.ImageMenu = menubar.addMenu('&Image')
        self.ProcessMenu = menubar.addMenu('&Process')
        self.AnalyseMenu = menubar.addMenu('&Analyse')
        self.AboutMenu = menubar.addMenu('&About')
        
        self.aboutAction=QAction(QtGui.QIcon(self.icon+"LOA.png'"), 'About', self)
        self.AboutMenu.addAction(self.aboutAction)
        self.aboutAction.triggered.connect(lambda:self.open_widget(self.aboutWidget) )     
        self.statusBar = QStatusBar()
        self.setContentsMargins(0, 0, 0, 0)
        
        self.setStatusBar(self.statusBar)
       
        
        self.vbox1=QVBoxLayout() 
        self.vbox1.setContentsMargins(0, 0, 0, 0)
        
        self.hbox0=QHBoxLayout()
        self.hbox0.setContentsMargins(0, 0, 0, 0)
        self.vbox1.addLayout(self.hbox0)
        
         
        self.openAct = QAction(QtGui.QIcon(self.icon+"Open.png"), 'Open File', self)
        self.openAct.setShortcut('Ctrl+o')
        self.openAct.triggered.connect(self.OpenF)
        self.toolBar.addAction(self.openAct)
        self.fileMenu.addAction(self.openAct)
        
        
        self.openActNewWin = QAction(QtGui.QIcon(self.icon+"Open.png"), 'Open in new window', self)
        #self.openActNewWin.setShortcut('Ctrl+o')
        self.openActNewWin.triggered.connect(self.OpenFNewWin)
        #self.toolBar.addAction(self.openActNewWin)
        self.fileMenu.addAction(self.openActNewWin)
        
        
        self.saveAct=QAction(QtGui.QIcon(self.icon+"disketteSave.png"), 'Save file', self)
        self.saveAct.setShortcut('Ctrl+s')
        self.saveAct.triggered.connect(self.SaveF)
        self.toolBar.addAction(self.saveAct)
        self.fileMenu.addAction(self.saveAct)

        
        self.checkBoxAutoSave=QAction(QtGui.QIcon(self.icon+"diskette.png"),'AutoSave Off',self)
        self.checkBoxAutoSave.setCheckable(True)
        self.checkBoxAutoSave.setChecked(False)
        
        self.checkBoxAutoSave.triggered.connect(self.autoSaveColor)
        self.toolBar.addAction(self.checkBoxAutoSave)
        self.fileMenu.addAction(self.checkBoxAutoSave)
        
        
        self.optionAutoSaveAct=QAction(QtGui.QIcon(self.icon+"Settings.png"),'Options',self) #
        self.optionAutoSaveAct.triggered.connect(lambda:self.open_widget(self.winOpt))
        self.toolBar.addAction(self.optionAutoSaveAct)
        self.fileMenu.addAction(self.optionAutoSaveAct)
        
        self.preferenceAct=QAction(QtGui.QIcon(self.icon+"pref.png"),'Preferences',self) #
        self.preferenceAct.triggered.connect(lambda:self.open_widget(self.winPref))
        
        self.fileMenu.addAction(self.preferenceAct)
        
        self.historyAct=QAction(QtGui.QIcon(self.icon+"time.png"),'&History',self) #
        self.historyAct.triggered.connect(lambda:self.open_widget(self.winHistory))
        self.fileMenu.addAction(self.historyAct)
        
        
        self.checkBoxPlot=QAction(QtGui.QIcon(self.icon+"target.png"),'Cross On (ctrl+b to block ctrl+d to unblock)',self)
        self.checkBoxPlot.setCheckable(True)
        self.checkBoxPlot.setChecked(False)
        self.checkBoxPlot.triggered.connect(self.PlotXY)
        self.toolBar.addAction(self.checkBoxPlot)
        self.AnalyseMenu.addAction(self.checkBoxPlot)
        
        self.maxGraphBox=QAction('Set Cross on the max',self)
        self.maxGraphBox.setCheckable(True)
        self.maxGraphBox.setChecked(False)
        self.maxGraphBox.triggered.connect(self.Coupe)
        self.AnalyseMenu.addAction(self.maxGraphBox)
        
        
        
        self.label_CrossValue=QLabel()
        self.label_CrossValue.setStyleSheet("font:13pt")
        
        self.label_Cross=QLabel()
        #self.label_Cross.setMaximumHeight(20)
        self.label_Cross.setMaximumWidth(170)
        self.label_Cross.setStyleSheet("font:12pt")
        self.statusBar.addPermanentWidget(self.label_Cross)
        self.statusBar.addPermanentWidget(self.label_CrossValue)
        
        self.labelFileName=QLabel("File :")
        self.labelFileName.setStyleSheet("font:8pt;")
        self.labelFileName.setMinimumHeight(30)
        self.labelFileName.setMaximumWidth(40)
        
        self.fileName=QLabel()
        self.fileName.setStyleSheet("font:10pt")
        self.fileName.setMaximumHeight(30)
        self.fileName.setMaximumWidth(200000)
        self.fileName.setAlignment(Qt.AlignmentFlag.AlignVCenter)
        self.statusBar.addWidget(self.labelFileName)
        self.statusBar.addWidget(self.fileName)
        
        self.labelFrameName=QLabel("Frame :")
        self.labelFrameName.setStyleSheet("font:8pt;")
        self.labelFrameName.setMinimumHeight(30)
        self.labelFrameName.setMaximumWidth(70)
        self.labelFrameName.setAlignment(Qt.AlignmentFlag.AlignVCenter)
        self.frameName=QLabel()
        self.frameName.setStyleSheet("font:8pt")
        self.frameName.setMaximumHeight(30)
        self.frameName.setMaximumWidth(100)
        self.frameName.setAlignment(Qt.AlignmentFlag.AlignVCenter)
        self.statusBar.addPermanentWidget(self.labelFrameName)
        self.statusBar.addPermanentWidget(self.frameName)
        
        self.ImgFrame=QToolButton()
        self.ImgFrame.setStyleSheet("QToolButton:!pressed{border-image: url(./icons/data1.png);background-color: transparent;border-color: green;}""QToolButton:pressed{image: url(./icons/data2.png) ;background-color: transparent;border-color: blue}")
        self.statusBar.addPermanentWidget(self.ImgFrame)
        
        self.checkBoxScale=QAction(QtGui.QIcon(self.icon+"expand.png"),' Auto Scale on',self)
        self.checkBoxScale.setCheckable(True)
        self.checkBoxScale.setChecked(True)
        self.toolBar.addAction(self.checkBoxScale)
        self.ImageMenu.addAction(self.checkBoxScale)
        self.checkBoxScale.triggered.connect(self.checkBoxScaleImage)
        
        self.checkBoxColor=QAction(QtGui.QIcon(self.icon+"colors-icon.png"),'Color on',self)
        self.checkBoxColor.triggered.connect(self.Color)
        self.checkBoxColor.setCheckable(True) 
        self.checkBoxColor.setChecked(True)
        self.toolBar.addAction(self.checkBoxColor)
        self.ImageMenu.addAction(self.checkBoxColor)
        
    
        self.checkBoxHist=QAction(QtGui.QIcon(self.icon+"colourBar.png"),'Show colour Bar',self)
        self.checkBoxHist.setCheckable(True)
        self.checkBoxHist.setChecked(False)
        self.checkBoxHist.triggered.connect(self.HIST)
        self.ImageMenu.addAction(self.checkBoxHist)
        
        #self.ColorBox=QMenu('&LookUp Table',self) #
        menuColor=QMenu('LookUp Table',self)
        menuColor.addAction('thermal',self.Setcolor)
        menuColor.addAction('flame',self.Setcolor)
        menuColor.addAction('yellowy',self.Setcolor)
        menuColor.addAction('bipolar',self.Setcolor)
        menuColor.addAction('spectrum',self.Setcolor)
        menuColor.addAction('cyclic',self.Setcolor)
        menuColor.addAction('viridis',self.Setcolor) 
        menuColor.addAction('inferno',self.Setcolor)
        menuColor.addAction('plasma',self.Setcolor)      
        menuColor.addAction('magma',self.Setcolor)            
        
        #self.ColorBox.setMenu(menuColor)
        self.ImageMenu.addMenu(menuColor)
        
        
        self.checkBoxBg=QAction('Background Substraction On',self)
        self.checkBoxBg.setCheckable(True)
        self.checkBoxBg.setChecked(False)
        self.ImageMenu.addAction(self.checkBoxBg)
        
        
        
        if self.encercled=="on":
            self.energyBox=QAction(QtGui.QIcon(self.icon+"coin.png"),'Energy Encercled',self)
            self.energyBox.setShortcut('Ctrl+e')
            self.AnalyseMenu.addAction(self.energyBox)
            self.energyBox.triggered.connect(self.Energ)
        
            
        if self.math=="on":
            self.mathButton=QAction(QtGui.QIcon(self.icon+"math.png"),'Math',self)
            self.mathButton.triggered.connect(lambda:self.open_widget(self.winMath))
            self.ProcessMenu.addAction(self.mathButton)
            self.winMath.emitApply.connect(self.newDataReceived)
        
        self.paletteupButton=QAction(QtGui.QIcon(self.icon+"user.png"),'Brightness +',self)
        #self.paletteupButton.setShortcut('+')
        self.ProcessMenu.addAction(self.paletteupButton)
        self.paletteupButton.triggered.connect(self.paletteup)
        
        self.palettedownButton=QAction(QtGui.QIcon(self.icon+"userM.png"),'Brightness -',self)
        #self.palettedownButton.setShortcut('-')
        self.ProcessMenu.addAction(self.palettedownButton)
        self.palettedownButton.triggered.connect(self.palettedown)
        
        self.paletteautoButton=QAction(QtGui.QIcon(self.icon+"robotics.png"),'Brightness auto ',self)
        #self.palettedautoButton.setShortcut('-')
        self.ProcessMenu.addAction(self.paletteautoButton)
        self.paletteautoButton.triggered.connect(self.paletteauto)
        
        self.contrastButton=QAction(QtGui.QIcon(self.icon+"ying-yang.png"),'Contrast 5%-95%',self)
        #self.palettedautoButton.setShortcut('-')
        self.ProcessMenu.addAction(self.contrastButton)
        self.contrastButton.triggered.connect(self.contrast)
        
        
        
        if self.winFilter=='on':
            #self.filtreBox=QAction('&Filters',self)
            self.menuFilter=QMenu('&Filters',self)
            self.menuFilter.addAction('&Gaussian',self.Gauss)
            self.menuFilter.addAction('&Median',self.Median)
            self.menuFilter.addAction('&Threshold',self.Threshold)
            self.menuFilter.addAction('&Origin',self.Orig)
            
            #self.filtreBox.setMenu(menu)
            self.ProcessMenu.addMenu(self.menuFilter)
        
        if self.plot3D=="on":
            self.box3d=QPushButton('3D', self)
            self.toolBar.addWidget(self.box3d)
        
        if self.meas=='on':
            self.MeasButton=QAction(QtGui.QIcon(self.icon+"laptop.png"),'Measure',self)
            self.MeasButton.setShortcut('ctrl+m')
            self.MeasButton.triggered.connect(self.Measurement)
            self.AnalyseMenu.addAction(self.MeasButton)
            
        if self.fft=='on':
            self.fftButton=QAction('FFT')
            self.AnalyseMenu.addAction(self.fftButton)
            self.fftButton.triggered.connect(self.fftTransform)
        
        self.PointingButton=QAction(QtGui.QIcon(self.icon+"recycle.png"),'Pointing',self)
        self.PointingButton.triggered.connect(self.Pointing)
        self.AnalyseMenu.addAction(self.PointingButton)       
        
        
        
        
        
        
        self.ZoomRectButton=QAction(QtGui.QIcon(self.icon+"loupe.png"),'Zoom',self)
        self.ZoomRectButton.triggered.connect(self.zoomRectAct)
        self.toolBar.addAction(self.ZoomRectButton)
        
        
        self.ligneButton=QAction(QtGui.QIcon(self.icon+"line.png"),'add  Line',self)
        self.toolBar.addAction(self.ligneButton)
        
        self.rectangleButton=QAction(QtGui.QIcon(self.icon+"rectangle.png"),'Add Rectangle',self)
        self.toolBar.addAction(self.rectangleButton)
        
        self.circleButton=QAction(QtGui.QIcon(self.icon+"Red_circle.png"),'add  Cercle',self)
        self.toolBar.addAction(self.circleButton)
        
        self.PlotButton=QAction(QtGui.QIcon(self.icon+"analytics.png"),'Plot Profile',self)
        self.PlotButton.triggered.connect(self.CUT)
        self.PlotButton.setShortcut("ctrl+k")
        self.AnalyseMenu.addAction(self.PlotButton)
        
        
        self.showMaxButton=QAction('Show max',self)
        self.showMaxButton.triggered.connect(self.ZoomMAX)
        self.winZoomMax=ZOOM()
        self.AnalyseMenu.addAction(self.showMaxButton)
        
        
        
        self.flipButton=QAction(QtGui.QIcon(self.icon+"fliphorizontal.png"),'Flip Horizontally',self)
        self.flipButton.setCheckable(True)
        self.flipButton.setChecked(False)
        self.flipButton.triggered.connect(self.flipAct)
        self.ImageMenu.addAction(self.flipButton)
        
        
        self.flipButtonVert=QAction(QtGui.QIcon(self.icon+"flipvertical.png"),'Flip Horizontally',self)
        self.flipButtonVert.setCheckable(True)
        self.flipButtonVert.setChecked(False)
        self.ImageMenu.addAction(self.flipButtonVert)
        self.flipButtonVert.triggered.connect(self.flipVertAct)
        
        
        self.vbox2=QVBoxLayout()
        
        self.winImage = pg.GraphicsLayoutWidget()
        self.winImage.setContentsMargins(0,0,0,0)
        self.winImage.setAspectLocked(True)
        self.winImage.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        self.winImage.ci.setContentsMargins(0,0,0,0)
        self.vbox2.addWidget(self.winImage)
        self.vbox2.setContentsMargins(0,0,0,0)
        
        self.p1=self.winImage.addPlot()
        self.imh=pg.ImageItem()
        self.axeX=self.p1.getAxis('bottom')
        self.axeY=self.p1.getAxis('left')
        self.p1.addItem(self.imh)
        self.p1.setMouseEnabled(x=False,y=False)
        self.p1.setContentsMargins(0,0,0,0)
   
        self.p1.setAspectLocked(True,ratio=1)
        self.p1.showAxis('right',show=False)
        self.p1.showAxis('top',show=False)
        self.p1.showAxis('left',show=False)
        self.p1.showAxis('bottom',show=False)
        
        if self.bloqKeyboard==True:
            self.vLine = pg.InfiniteLine(angle=90, movable=False,pen='r')
            self.hLine = pg.InfiniteLine(angle=0, movable=False,pen='r')
        else:
            self.vLine = pg.InfiniteLine(angle=90, movable=False,pen='y')
            self.hLine = pg.InfiniteLine(angle=0, movable=False,pen='y')

        self.xc=int(self.conf.value(self.name+"/xc"))
        self.yc=int(self.conf.value(self.name+"/yc"))
        self.rx=int(self.conf.value(self.name+"/rx"))
        self.ry=int(self.conf.value(self.name+"/ry"))
        self.vLine.setPos(self.xc)
        self.hLine.setPos(self.yc)
       
        
        self.ro1=pg.EllipseROI([self.xc,self.yc],[self.rx,self.ry],pen='r',movable=False)
        self.ro1.setPos([self.xc-(self.rx/2),self.yc-(self.ry/2)])
        
       
        self.roiFluence=pg.EllipseROI([self.xc,self.yc],[self.rx,self.ry],pen='b',movable=True)
        self.roiFluence.setPos([self.xc-(self.rx/2),self.yc-(self.ry/2)])
       
        
        # text for fwhm on p1
        self.textX = pg.TextItem(angle=-90) 
        self.textY = pg.TextItem()
        
        #histogram
        self.hist = pg.HistogramLUTItem() 
        self.hist.setImageItem(self.imh)
        self.hist.autoHistogramRange()
        self.hist.gradient.loadPreset('flame')
        
        ##  XY  graph
        self.curve2=pg.PlotCurveItem()
        self.curve3=pg.PlotCurveItem()
        
        ## slider to open multi file
        self.sliderImage=QSlider(Qt.Horizontal)
        self.sliderImage.setEnabled(False)
        self.vbox2.addWidget(self.sliderImage)
        
        ## main layout
        
        # vMainLayout=QVBoxLayout()
        
        hMainLayout=QHBoxLayout()
        # vMainLayout.addLayout(self.hboxBar)
        #vMainLayout.addLayout(hMainLayout)
        
        if self.aff=='right':
            hMainLayout.addLayout(self.vbox2)
            hMainLayout.addLayout(self.vbox1)
        if self.aff=='left':
            hMainLayout.addLayout(self.vbox1)
            hMainLayout.addLayout(self.vbox2)
            
        
        
        hMainLayout.setContentsMargins(1,1,1,1)
        #hMainLayout.setSpacing(1)
        #hMainLayo.setStretch(10,1)
        MainWidget=QFrame()#
        
        if self.color is not None :
            
            self.winImage.setStyleSheet("border : 3px solid  %s" % self.color);
        
        MainWidget.setLayout(hMainLayout)
        
        self.setCentralWidget(MainWidget)
        #self.setContentsMargins(1,1,1,1)
        #self.plotLine=pg.LineSegmentROI(positions=((self.dimx/2-100,self.dimy/2),(self.dimx/2+100,self.dimy/2)), movable=True,angle=0,pen='b')
        self.plotLine=pg.LineSegmentROI(positions=((0,200),(200,200)), movable=True,angle=0,pen='w')
        #self.plotLine=pg.PolyLineROI(positions=((0,200),(200,200),(300,200)), movable=True,angle=0,pen='w')
        self.plotRect=pg.RectROI([self.xc,self.yc],[4*self.rx,self.ry],pen='g')
        self.plotCercle=pg.CircleROI([self.xc,self.yc],[80,80],pen='g')
        self.plotRectZoom=pg.RectROI([self.xc,self.yc],[4*self.rx,self.ry],pen='w')
        self.plotRectZoom.addScaleHandle((0,0),center=(1,1))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6')) # dark style
        
        
        
    def actionButton(self):
        # action of button
        
       
        self.ro1.sigRegionChangeFinished.connect(self.roiChanged)
        self.ligneButton.triggered.connect(self.LIGNE)
        self.rectangleButton.triggered.connect(self.Rectangle)
        self.circleButton.triggered.connect(self.CERCLE)
        
        self.plotLine.sigRegionChangeFinished.connect(self.LigneChanged)
        self.plotRect.sigRegionChangeFinished.connect(self.RectChanged)
        self.plotCercle.sigRegionChangeFinished.connect(self.CercChanged)
        
        # if self.plot3D=="on":
        #     self.box3d.clicked.connect(self.Graph3D)
            
        self.winPref.closeEventVar.connect(self.ScaleImg)
        
        
        self.sliderImage.valueChanged.connect(self.SliderImgFct)
        # self.dockImage.topLevelChanged.connect(self.fullScreen)
        self.roiFluence.sigRegionChangeFinished.connect(self.fluenceFct)
        
        if self.parent is not None:
            self.parent.signalData.connect(self.newDataReceived)
        
    def fullScreen(self):
        if  self.fullscreen==False:
            self.fullscreen=True
            self.dockImage.showMaximized()
        else :
            self.fullscreen=False
            self.dockImage.showNormal()
        
    def shortcut(self):
        # keyboard shortcut
        
        self.shortcutPu=QShortcut(QtGui.QKeySequence("+"),self)
        self.shortcutPu.activated.connect(self.paletteup)
        self.shortcutPu.setContext(Qt.ShortcutContext(3))
        #3: The shortcut is active when its parent widget, or any of its children has focus. default O The shortcut is active when its parent widget has focus.
        self.shortcutPd=QtGui.QShortcut(QtGui.QKeySequence("-"),self)
        self.shortcutPd.activated.connect(self.palettedown)
        self.shortcutPd.setContext(Qt.ShortcutContext(3))
        
        
        self.shortcutBloq=QtGui.QShortcut(QtGui.QKeySequence("Ctrl+b"),self)
        self.shortcutBloq.activated.connect(self.bloquer)
        self.shortcutBloq.setContext(Qt.ShortcutContext(3))
        
        self.shortcutDebloq=QtGui.QShortcut(QtGui.QKeySequence("Ctrl+d"),self)
        self.shortcutDebloq.activated.connect(self.debloquer)
        self.shortcutDebloq.setContext(Qt.ShortcutContext(3))
        
        
        
        # mousse action:
        self.proxy=pg.SignalProxy(self.p1.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.p1.scene().sigMouseClicked.connect(self.mouseClick)
        self.vb=self.p1.vb
        
        
    def Energ(self):
        # open windget for calculated encercled energy and plot it vs shoot number
        self.open_widget(self.winEncercled)
        self.winEncercled.Display(self.data)
        
    def LIGNE(self) : 
        #plot a line
        try :
            self.p1.removeItem(self.plotRect)
            self.p1.removeItem(self.plotCercle)
        except:pass
    
        if self.ite=='line':
            self.p1.removeItem(self.plotLine)
            self.ite=None
        else: 
            self.ite='line'
            self.p1.addItem(self.plotLine)
            self.LigneChanged()
        
      
    def LigneChanged(self):
        # take ROI 
        self.cut=self.plotLine.getArrayRegion(self.data,self.imh)
        
        if self.winPref.checkBoxAxeScale.isChecked()==1:
            self.linePoints=self.plotLine.listPoints()
            self.lineXo=self.linePoints[0][0]
            self.lineYo=self.linePoints[0][1]
            self.lineXf=self.linePoints[1][0]
            self.lineYf=self.linePoints[1][1]
            #self.plotLineAngle=np.arctan((self.lineYf-self.lineYo)/(self.lineXf-self.lineXo))
            # print('angle',self.plotLineAngle*360/(2*3.14),self.cut.size)
            
            step=(self.winPref.stepX**2*(self.lineXo-self.lineXf)**2+self.winPref.stepY**2*(self.lineYo-self.lineYf)**2)
            step=step**0.5/self.cut.size
            self.absiLine=np.arange(0,(self.cut.size)*step,step)
            
            #print(self.absiLine)
        
    def Rectangle(self)  :
        
        try :
            self.p1.removeItem(self.plotLine)
            self.p1.removeItem(self.plotCercle)
        except:pass
        
        if self.ite=='rect':
            self.p1.removeItem(self.plotRect)
            self.ite=None
        else :
            self.p1.addItem(self.plotRect)
            self.plotRect.setPos([self.dimx/2,self.dimy/2])
            self.ite='rect'
            self.RectChanged()
        
    def RectChanged(self):
        # take ROI
        self.cut=(self.plotRect.getArrayRegion(self.data,self.imh))
        if self.winPref.plotRectOpt.currentIndex()==0:
            self.cut1=self.cut.mean(axis=1)
        else:
            self.cut1=self.cut.sum(axis=1)
        
    def CERCLE(self) : 
        try :
            self.p1.removeItem(self.plotRect)
            self.p1.removeItem(self.plotLine)
        except:pass
        #self.p1.clear()
        if self.ite=='cercle':
            self.p1.removeItem(self.plotCercle)
            self.ite=None
        else:
            self.p1.addItem(self.plotCercle) 
            self.plotCercle.setPos([self.dimx/2,self.dimy/2])
            self.ite='cercle'
        
    def CercChanged(self):
        # take ROI
        self.cut=(self.plotCercle.getArrayRegion(self.data,self.imh))
        self.cut1=self.cut.mean(axis=1)
    
    def CUT(self): 
        # plot on a separated widget the ROI plot profile
        if self.ite=='line':
            self.signalTrans['data']=self.cut
            self.open_widget(self.winCoupe)
            if self.winPref.checkBoxAxeScale.isChecked()==1:
                
                self.signalTrans['axis']=self.absiLine
                self.signalPlot.emit(self.signalTrans)
                # self.winCoupe.PLOT(self.cut,axis=self.absiLine)
            else:
                # self.winCoupe.PLOT(self.cut)#,symbol=False)
                self.signalPlot.emit(self.signalTrans)
        if self.ite=='rect':
            self.signalTrans['data']=self.cut1
            self.open_widget(self.winCoupe)
            # self.winCoupe.PLOT(self.cut1)#,symbol=False)
            self.signalPlot.emit(self.signalTrans)
   
    # def Graph3D (self):
        
    #     self.open_widget(self.Widget3D)
    #     self.Widget3D.Plot3D(self.data)
        
        
    def Measurement(self) :
        # show widget for measurement on all image or ROI  (max, min mean ...)
        if self.ite=='rect':
            self.RectChanged()
            if self.meas=="on":
                self.winM.setFile(self.nomFichier)
                self.open_widget(self.winM)
                self.signalMeas.emit(self.cut)
                # self.winM.Display(self.cut)
            
        if self.ite=='cercle':
            self.CercChanged()
            if self.meas=="on":
                self.winM.setFile(self.nomFichier)
                self.open_widget(self.winM)
                self.signalMeas.emit(self.cut)
                # self.winM.Display(self.cut)
        
        # if self.ite=='line':
        #     self.LigneChanged()
        #     self.winM.setFile(self.nomFichier)
        #     self.open_widget(self.winM)
        #     self.winM.Display(self.cut)
        
        if self.ite==None:
            if self.meas=="on":
                self.winM.setFile(self.nomFichier)
                self.open_widget(self.winM)
                
                self.signalMeas.emit(self.data)
                
                # self.winM.Display(self.data)
                
    def Pointing(self) :

        self.open_widget(self.winPointing)

        if self.ite=='rect':
            self.RectChanged()
            pData=self.cut
        elif self.ite=='cercle':
            self.CercChanged() 
            pData=self.cut
        elif self.ite==None:
            pData=self.data
        else :pData=self.data


        if self.winPref.checkBoxAxeScale.isChecked()==1:
            self.signalPointing.emit(pData,self.winPref.stepX,self.winPref.stepX)
                #self.winPointing.Display(pData,self.winPref.stepX,self.winPref.stepX)
        else:
            self.signalPointing.emit(pData)
            # self.winPointing.Display(pData)

    def fftTransform(self):
        # show on a new widget fft 
        if self.ite=='rect':
            self.RectChanged()
            self.open_widget(self.winFFT)
            self.winFFT.Display(self.cut)   
        if self.ite=='cercle':
            self.CercChanged()
            self.open_widget(self.winFFT)
            self.winFFT.Display(self.cut)
        if self.ite=='line':
            self.LigneChanged()
            self.open_widget(self.winFFT1D)
            if self.cut.ndim==1:
                datafft=np.fft.fft(np.array(self.cut))
                self.norm=abs(np.fft.fftshift(datafft))
                self.norm=np.log10(1+self.norm)
                self.winFFT1D.PLOT(self.norm)
            
        if self.ite==None:
            self.open_widget(self.winFFT)
            self.winFFT.Display(self.data) 
        

        
    @pyqtSlot (object)   
    def Display(self,data):
        #  display the data and refresh all the calculated things and plots
        self.data=data
        
        
        if self.checkBoxBg.isChecked()==True and self.winOpt.dataBgExist==True:
            try :
                self.data=self.data-self.winOpt.dataBg
            except :
                msg = QMessageBox()
                msg.setIcon(QMessageBox.Icon.Critical)
                msg.setText("Background not soustracred !")
                msg.setInformativeText("Background file error  ")
                msg.setWindowTitle("Warning ...")
                msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
                msg.exec_()
                
        if self.checkBoxBg.isChecked()==True and self.winOpt.dataBgExist==False:
                msg = QMessageBox()
                msg.setIcon(QMessageBox.Icon.Critical)
                msg.setText("Background not soustracted !")
                msg.setInformativeText("Background file not selected in options menu ")
                msg.setWindowTitle("Warning ...")
                msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
                
                msg.exec_()
                
        
        #### filtre
        if self.filter=='gauss':
            self.data=gaussian_filter(self.data,self.sigma)
            #print('gauss filter')
            
        if self.filter=='median':
            self.data=median_filter(self.data,size=self.sigma)
            #print('median filter')
        if self.filter=='threshold':
            self.data=np.where( self.data<self.threshold,0,self.data)
        #### fluence 
        if self.winPref.checkBoxFluence.isChecked()==1: # fluence on 
           energy=self.winPref.energy.value()
           #print('energy',energy)
           if self.winPref.checkBoxAxeScale.isChecked()==0:  # en pixel
               size=1#self.sizeFluenceX*self.sizeFluenceY
               self.labelValue=' mJ/pixel2'
           if self.winPref.checkBoxAxeScale.isChecked()==1: #en micron
               size=1E-8*self.winPref.stepX*self.winPref.stepY
               self.labelValue=' mJ/cm2'
               
           enrgTot=self.roiFluence.getArrayRegion(self.data,self.imh).sum()
           
           self.data=1000*(self.data*energy/enrgTot)/size #(microJ/cm2)
        # self.p1.setAspectLocked(True,ratio=1)
        else :
            self.labelValue=''
            
        #### color  and sacle 
        if self.checkBoxScale.isChecked()==1: # color autoscale on
            
            if self.winPref.checkBoxAxeScale.isChecked()==1:
                self.axeX.setScale(self.winPref.stepX)
                self.axeY.setScale(self.winPref.stepY)
                self.axeX.setLabel('um')
                self.axeY.setLabel('um')
                self.axeX.showLabel(True)
            if self.winPref.checkBoxAxeScale.isChecked()==0:
                self.scaleAxis="off"
                self.axeX.setScale(1)
                self.axeY.setScale(1)  
                self.axeX.showLabel(False)
            self.imh.setImage(self.data,autoLevels=True,autoDownsample=True) #.astype(float)
        else :
            self.imh.setImage(self.data,autoLevels=False,autoDownsample=True)
        
        
        ####update    
        self.Coupe()#self.PlotXY() # graph update
        self.zoomRectupdate() # update rect  
        
        if self.encercled=="on":
            if self.winEncercled.isWinOpen==True:
                self.signalEng.emit(self.data)
                # self.winEncercled.Display(self.data) ## energy update
        
        if self.winCoupe.isWinOpen==True:
            if self.ite=='line':
                self.LigneChanged()
                self.CUT()
            if self.ite=='rect':
                self.RectChanged()
                self.CUT()
            if self.ite=='cercle':
                self.CercChanged()
        if self.meas=="on":       
            if self.winM.isWinOpen==True: #  measurement update
                if self.ite=='rect':
                    self.RectChanged()
                    self.Measurement()
                elif self.ite=='cercle':
                    self.CercChanged()
                    self.Measurement()
                else :
                    self.Measurement()
        if self.fft=='on':        
            if self.winFFT.isWinOpen==True: # fft update
                self.winFFT.Display(self.data)
                
        # if self.plot3D=="on":
        #     if self.Widget3D.isWinOpen==True:
        #         self.Graph3D()
        if self.winPointing.isWinOpen==True:
            self.Pointing()
        if self.winZoomMax.isWinOpen==True:
            self.ZoomMAX()   
            
        #### autosave
        if self.checkBoxAutoSave.isChecked()==True: ## autosave data
            self.pathAutoSave=str(self.conf.value(self.name+'/pathAutoSave'))
            self.fileNameSave=str(self.conf.value(self.name+'/nameFile'))
            date=time.strftime("%Y_%m_%d_%H_%M_%S")
            self.numTir=int(self.conf.value(self.name+'/tirNumber'))
            # if self.numTir<10:
            #     num="00"+str(self.numTir)
            # elif 9<self.numTir<100:
            #     num="0"+str(self.numTir)
            # else:
            num=str(self.numTir)
            if self.winOpt.checkBoxDate.isChecked()==True: # add the date
                nomFichier=str(str(self.pathAutoSave)+'/'+self.fileNameSave+'_'+num+'_'+date)
            else :
                nomFichier=str(str(self.pathAutoSave)+'/'+self.fileNameSave+'_'+num)

            print( nomFichier, 'saved')
            if self.winOpt.checkBoxTiff.isChecked()==True: #save as tiff 
                self.dataS=self.dataS=np.rot90(self.data,1)
                img_PIL = Image.fromarray(self.dataS)
                img_PIL.save(str(nomFichier)+'.TIFF',format='TIFF') 
            else :
                
                np.savetxt(str(nomFichier)+'.txt',self.data)
                
            if self.winOpt.checkBoxServer.isChecked()==False: # if not connected to server we had +1
                self.numTir+=1
                self.winOpt.setTirNumber(self.numTir)
            self.conf.setValue(self.name+"/tirNumber",self.numTir)
                
            self.fileName.setText(nomFichier)
    
        
        
        
    def mouseClick(self,evt): # block the cross if mousse button clicked
        
#            
#            
#        else:
        if self.bloqq==1:
            self.bloqq=0
            
        else :
            self.bloqq=1
#            self.conf.setValue(self.name+"/xc",int(self.xc)) # save cross postion in ini file
#            self.conf.setValue(self.name+"/yc",int(self.yc))
        
            
    def mouseMoved(self,evt):
        if self.checkBoxPlot.isChecked()==False or self.bloqKeyboard==True :  # if not  cross or crossblocked by  keyboard: 
            
            if self.bloqq==0:
                
                pos = evt[0]  ## using signal proxy turns original arguments into a tuple
                if self.p1.sceneBoundingRect().contains(pos):
                    
                    mousePoint = self.vb.mapSceneToView(pos)
                    self.xMouse = (mousePoint.x())
                    self.yMouse= (mousePoint.y())
                    if ((self.xMouse>0 and self.xMouse<self.dimx-1) and (self.yMouse>0 and self.yMouse<self.dimy-1) ):
                        self.xc = self.xMouse
                        self.yc= self.yMouse  
                        
                        try :
                            dataCross=self.data[int(self.xc),int(self.yc)]
                    
                        except :
                            dataCross=0  # evoid to have an error if cross if out of the image
                            self.xc=0
                            self.yc=0
                        if self.winPref.checkBoxAxeScale.isChecked()==1: # scale axe on 
                            self.label_Cross.setText('x='+ str(round(int(self.xc)*self.winPref.stepX,2)) + '  um'+' y=' + str(round(int(self.yc)*self.winPref.stepY,2)) +' um')
                        else : 
                            
                            self.label_Cross.setText('x='+ str(int(self.xc)) + ' y=' + str(int(self.yc)) )
                        
                        dataCross=round(dataCross,3) # take data  value  on the cross
                        self.label_CrossValue.setText(' v.=' + str(dataCross)+self.labelValue)
        ## the cross mouve with the mousse mvt
        if self.bloqKeyboard==False :  #mouse not  blocked by  keyboard
            if self.bloqq==0: # mouse not  blocked by mouse  click
                
                pos = evt[0]  ## using signal proxy turns original arguments into a tuple
                if self.p1.sceneBoundingRect().contains(pos):
                    
                    mousePoint = self.vb.mapSceneToView(pos)
                    self.xMouse = (mousePoint.x())
                    self.yMouse= (mousePoint.y())
                    if ((self.xMouse>0 and self.xMouse<self.dimx-1) and (self.yMouse>0 and self.yMouse<self.dimy-1) ):
                            self.xc = self.xMouse
                            self.yc= self.yMouse  
                            self.vLine.setPos(self.xc)
                            self.hLine.setPos(self.yc) # the cross move only in the graph    
                            if self.roiCross==True :
                                self.ro1.setPos([self.xc-(self.rx/2),self.yc-(self.ry/2)])
                            self.PlotXY()
                
    def fwhm(self,x, y, order=3):
        """
            Determine full-with-half-maximum of a peaked set of points, x and y.
    
        """
        y=gaussian_filter(y,5) # filtre for reducing noise
        half_max = np.amax(y)/2.0
        s = splrep(x, y - half_max,k=order) # F
        roots = sproot(s) # Given the knots .
        if len(roots) > 2:
            pass
           
        elif len(roots) < 2:
            pass
        else:
            return np.around(abs(roots[1] - roots[0]),decimals=2)
        
       
    def Coupe(self):
        # make  plot profile on cross
        
        if self.checkBoxPlot.isChecked()==True:
            
            if self.maxGraphBox.isChecked()==True  and self.bloqKeyboard==False  : # find and fix the cross on the maximum of the image
                
                #dataF=gaussian_filter(self.data,3)
                # dataF=self.data
                (self.xc,self.yc)=np.unravel_index(self.data.argmax(),self.data.shape) #take the max ndimage.measurements.center_of_mass(dataF)#
                self.vLine.setPos(self.xc)
                self.hLine.setPos(self.yc)
                if self.roiCross==True:
                    self.ro1.setPos([self.xc-(self.rx/2),self.yc-(self.ry/2)])
            
                
            try :
                dataCross=self.data[int(self.xc),int(self.yc)]
                
            except :
                dataCross=0  # evoid to have an error if cross if out of the image
                self.xc=0
                self.yc=0
               
            coupeX=self.data[int(self.xc),:]
            coupeY=self.data[:,int(self.yc)]
            xxx=np.arange(0,int(self.dimx),1)
            yyy=np.arange(0,int(self.dimy),1)
            coupeXMax=np.max(coupeX)
            coupeYMax=np.max(coupeY)
            
            
            if coupeXMax==0: # avoid zero
                coupeXMax=1
            
            if coupeYMax==0:
                coupeYMax=1
                
            if self.winPref.checkBoxAxeScale.isChecked()==1: # scale axe on 
                self.label_Cross.setText('x='+ str(round(int(self.xc)*self.winPref.stepX,2)) + '  um'+' y=' + str(round(int(self.yc)*self.winPref.stepY,2)) +' um')
            else : 
                self.label_Cross.setText('x='+ str(int(self.xc)) + ' y=' + str(int(self.yc)) )
                
            dataCross=round(dataCross,3) # take data  value  on the cross
            
            self.label_CrossValue.setText(' v.=' + str(dataCross))
            
            coupeXnorm=(self.data.shape[0]/10)*(coupeX/coupeXMax)
            coupeYnorm=(self.data.shape[1]/10)*(coupeY/coupeYMax)
            
            if self.plotRectZoomEtat=="ZoomOut": # the cut line follow the zoom
               
                self.curve2.setData(20+self.xZoomMin+coupeXnorm,yyy,clear=True)
                self.curve3.setData(xxx,20+self.yZoomMin+coupeYnorm,clear=True)
            
            else :
                 # normalize the curves
                self.curve2.setData(20+self.xminR+coupeXnorm,yyy,clear=True)
                self.curve3.setData(xxx,20+self.yminR+coupeYnorm,clear=True)
                
            
            
            ###  fwhm on the  X et Y curves if max  >20 counts if checked in winOpt
            
            
            if self.winPref.checkBoxFwhm.isChecked()==1: # show fwhm values on graph
                xCXmax=np.amax(coupeXnorm) # max
                if xCXmax>20:
                    try :
                        fwhmX=self.fwhm(yyy, coupeXnorm, order=3)
                    except : fwhmX=None
                    if fwhmX==None:
                        self.textX.setText('')
                    else:
                        if self.winPref.checkBoxAxeScale.isChecked()==1:
                            self.textX.setText('fwhm='+str(round(fwhmX*self.winPref.stepX,2))+' um',color='w')
                        else :
                           self.textX.setText('fwhm='+str(round(fwhmX,2)),color='w')
                    yCXmax=yyy[coupeXnorm.argmax()]
                    
                    self.textX.setPos(xCXmax+70,yCXmax+60)
                
                yCYmax=np.amax(coupeYnorm) # max
                
                if yCYmax>20:
                    try:
                        fwhmY=self.fwhm(xxx, coupeYnorm, order=3)
                    except :fwhmY=None
                    xCYmax=xxx[coupeYnorm.argmax()]
                    if fwhmY==None:
                        self.textY.setText('',color='w')
                    else:
                        if self.winPref.checkBoxAxeScale.isChecked()==1:
                            self.textY.setText('fwhm='+str(round(fwhmY*self.winPref.stepY,2))+' um',color='w')
                        else:
                            self.textY.setText('fwhm='+str(round(fwhmY,2)),color='w')
                            
                    self.textY.setPos(xCYmax-60,yCYmax+70)   
        
        if self.checkBoxPlot.isChecked()==False: ## write mouse value and not cross  value
            
            try :
                dataCross=self.data[int(self.xc),int(self.yc)]
                
            except :
                dataCross=0  # evoid to have an error if mousse is out of the image
                self.xc=0
                self.yc=0
                
            if self.winPref.checkBoxAxeScale.isChecked()==1: # scale axe on 
                self.label_Cross.setText('x='+ str(round(int(self.xc)*self.winPref.stepX,2)) + '  um'+' y=' + str(round(int(self.yc)*self.winPref.stepY,2)) +' um')
            else : 
                        
                self.label_Cross.setText('x='+ str(int(self.xc)) + ' y=' + str(int(self.yc)) )
                        
            dataCross=round(dataCross,3) # take data  value  on the mousse
            self.label_CrossValue.setText(' v.=' + str(dataCross)+self.labelValue)
 
    def PlotXY(self): # plot curves on the  graph
        
        if self.checkBoxPlot.isChecked()==1:
            
            self.p1.addItem(self.vLine, ignoreBounds=False)
            self.p1.addItem(self.hLine, ignoreBounds=False)
            self.p1.addItem(self.curve2)
            self.p1.addItem(self.curve3)
            self.p1.showAxis('left',show=True)
            self.p1.showAxis('bottom',show=True)
            self.p1.addItem(self.textX)
            self.p1.addItem(self.textY)
            if self.roiCross==True:
                self.p1.addItem(self.ro1)
            self.Coupe()
        else:
            self.p1.removeItem(self.vLine)
            self.p1.removeItem(self.hLine)
            self.p1.removeItem(self.curve2)
            self.p1.removeItem(self.curve3)
            self.p1.removeItem(self.textX)
            self.p1.removeItem(self.textY)
            self.p1.showAxis('left',show=False)
            self.p1.showAxis('bottom',show=False)
            #self.label_Cross.setText('')
            #self.label_CrossValue.setText('')
            if self.roiCross==True:
                self.p1.removeItem(self.ro1)

            
    def paletteup(self):
        # change the color scale
        levels=self.imh.getLevels()
        if levels[0]==None:
            xmax =self.data.max()
            xmin=self.data.min()
        else :
            xmax=levels[1]
            xmin=levels[0]
            
        self.imh.setLevels([xmin, xmax-(xmax- xmin) / 10])
        #hist.setImageItem(imh,clear=True)
        self.hist.setHistogramRange(xmin,xmax)

    def palettedown(self):
        
        levels=self.imh.getLevels()
        if levels[0]==None:
            xmax=self.data.max()
            xmin=self.data.min()
        else :
            xmax=levels[1]
            xmin=levels[0]
            
        self.imh.setLevels([xmin, xmax+ (xmax- xmin) / 10])
        #hist.setImageItem(imh,clear=True)
        self.hist.setHistogramRange(xmin,xmax)
    
    def paletteauto(self):
        
        xmax=self.data.max()
        xmin=self.data.min()
        if xmax==xmin:
            xmax=xmin+1
 
        self.imh.setLevels([xmin, xmax])
        self.hist.setHistogramRange(xmin,xmax)
    
    def contrast(self):
        if self.ite=='rect':
            self.cont=(self.plotRect.getArrayRegion(self.data,self.imh))
            xmax=self.cont.max()
            xmin=self.cont.min()
            
        else:
             xmax=self.data.max()
             xmin=self.data.min()
        xmin=0.05*xmax
        xmax=0.95*xmax
        self.imh.setLevels([xmin, xmax])
        self.hist.setHistogramRange(xmin,xmax)
  
     
    
    def Setcolor(self):
        action = self.sender()
        self.colorBar=str(action.text())
        
        self.hist.gradient.loadPreset(self.colorBar)
    
    def Color(self):
        """ image in colour/n&b
        """
        
        
        if self.checkBoxColor.isChecked()==1:
            self.checkBoxColor.setIcon(QtGui.QIcon(self.icon+"colors-icon.png"))
            self.hist.gradient.loadPreset(self.colorBar)
            self.checkBoxColor.setText('Color On')
        else:
            self.hist.gradient.loadPreset('grey')
            self.checkBoxColor.setIcon(QtGui.QIcon(self.icon+"circleGray.png"))
            self.checkBoxColor.setText('Gray')
            
    def roiChanged(self):
        
        self.rx=self.ro1.size()[0]
        self.ry=self.ro1.size()[1]
        self.conf.setValue(self.name+"/rx",int(self.rx))
        self.conf.setValue(self.name+"/ry",int(self.ry))
      
        
    def bloquer(self): # block the cross by keyboard
        
        self.bloqKeyboard=bool(True)
        self.conf.setValue(self.name+"/xc",int(self.xc))# save cross postion in ini file
        self.conf.setValue(self.name+"/yc",int(self.yc))
        self.conf.setValue(self.name+"/bloqKeyboard",bool(self.bloqKeyboard))
        self.vLine.setPen('r')
        self.hLine.setPen('r')
        self.ro1.setPen('r')
        
    def debloquer(self): # unblock the cross
        self.bloqKeyboard=bool(False)
        self.vLine.setPen('y')
        self.hLine.setPen('y')
        self.ro1.setPen('y')
        self.conf.setValue(self.name+"/bloqKeyboard",bool(self.bloqKeyboard))
        
        
        
    def HIST(self):
        #show histogramm
        if self.checkBoxHist.isChecked()==1:
            self.winImage.addItem(self.hist)
        else:
            self.winImage.removeItem(self.hist)
    
    
    def Gauss(self):
        # gauss filter
        self.filter='gauss'
        sigma, ok=QInputDialog.getInt(self,'Gaussian Filter ','Enter sigma value (radius)')
        if ok:
            self.sigma=sigma
            self.menuFilter.setTitle('F: Gaussian')
            self.Display(self.data)
        
        
    def Median(self):
        #median  filter
        self.filter='median'
        sigma, ok=QInputDialog.getInt(self,'Median Filter ','Enter sigma value (radius)')
        if ok:
            self.sigma=sigma
            self.menuFilter.setTitle('F: Median')
            self.Display(self.data)
        
    def Threshold(self):
        self.filter='threshold'
        threshold, ok=QInputDialog.getInt(self,'Threshold Filter ','Enter thresold value')
        if ok:
            self.threshold=threshold
            self.menuFilter.setTitle('F: Thresold')
            self.Display(self.data)
    def Orig(self):
        """
        return data without filter
        """
        self.data=self.dataOrg
        self.filter='origin'
        self.Display(self.data)
        self.menuFilter.setTitle('Filters')
        #print('original data')
        
    def OpenF(self,fileOpen=False):
        #open file in txt spe TIFF sif jpeg png  format
        fileOpen=fileOpen
        
        if fileOpen==False:
            
            chemin=self.conf.value(self.name+"/path")
            fname=QFileDialog.getOpenFileNames(self,"Open File",chemin,"Images (*.txt *.spe *.TIFF *.sif *.tif);;Text File(*.txt);;Ropper File (*.SPE);;Andor File(*.sif);; TIFF file(*.TIFF)")
           
            self.openedFiles=fname[0]
            
            self.nbOpenedImage=len(self.openedFiles)
            
            if self.nbOpenedImage==1:
                fichier=self.openedFiles[0]
                self.sliderImage.setEnabled(False)
                
            if self.nbOpenedImage>1:
                fichier=self.openedFiles[0]
                self.sliderImage.setMinimum(0)
                self.sliderImage.setMaximum(self.nbOpenedImage - 1)
                self.sliderImage.setValue(0)
                self.sliderImage.setEnabled(True)
                
                
        else:
            fichier=str(fileOpen)
                
        ext=os.path.splitext(fichier)[1]
        
        if ext=='.txt': # text file
            data=np.loadtxt(str(fichier))
        elif ext=='.spe' or ext=='.SPE': # SPE file
            dataSPE=SpeFile(fichier)
            data1=dataSPE.data[0]#.transpose() # first frame
            data=data1#np.flipud(data1)
        elif ext=='.TIFF' or ext=='.tif' or ext=='.Tiff' or ext=='.jpg' or ext=='.JPEG' or ext=='.png': # tiff File
            dat=Image.open(fichier)
            data=np.array(dat)
            data=np.rot90(data,3)
        elif ext=='.sif': 
            sifop=SifFile()
            im=sifop.openA(fichier)
            data=np.rot90(im,3)
#            self.data=self.data[250:495,:]
        else :
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Critical)
            msg.setText("Wrong file format !")
            msg.setInformativeText("The format of the file must be : .SPE  .TIFF .sif  png jpeg or .txt ")
            msg.setWindowTitle("Warning ...")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec_()
            
        chemin=os.path.dirname(fichier)
        self.conf.setValue(self.name+"/path",chemin)
        self.conf.setValue(self.name+"/lastFichier",os.path.split(fichier)[1])
        print('Open file  :  ',fichier)
        
        self.fileName.setText(str(fichier))
        self.nomFichier=os.path.split(fichier)[1]
        
        self.winHistory.Display(fichier)
        
        self.newDataReceived(data)
        
    
    def OpenFNewWin(self):
        
        chemin=self.conf.value(self.name+"/path")
        fname=QFileDialog.getOpenFileNames(self,"Open File",chemin,"Images (*.txt *.spe *.TIFF *.sif *.tif);;Text File(*.txt);;Ropper File (*.SPE);;Andor File(*.sif);; TIFF file(*.TIFF)")
           
            
        self.openedFiles=fname[0]
            
            
        fichier=self.openedFiles[0]
                
       
                
        ext=os.path.splitext(fichier)[1]
        
        if ext=='.txt': # text file
            data=np.loadtxt(str(fichier))
        elif ext=='.spe' or ext=='.SPE': # SPE file
            dataSPE=SpeFile(fichier)
            data1=dataSPE.data[0]#.transpose() # first frame
            data=data1#np.flipud(data1)
        elif ext=='.TIFF' or ext=='.tif' or ext=='.Tiff' or ext=='.jpg' or ext=='.JPEG' or ext=='.png': # tiff File
            dat=Image.open(fichier)
            data=np.array(dat)
            data=np.rot90(data,3)
        elif ext=='.sif': 
            sifop=SifFile()
            im=sifop.openA(fichier)
            data=np.rot90(im,3)
#            self.data=self.data[250:495,:]
        else :
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Critical)
            msg.setText("Wrong file format !")
            msg.setInformativeText("The format of the file must be : .SPE  .TIFF .sif  png jpeg or .txt ")
            msg.setWindowTitle("Warning ...")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec_()
            
        chemin=os.path.dirname(fichier)
        self.conf.setValue(self.name+"/path",chemin)
        self.conf.setValue(self.name+"/lastFichier",os.path.split(fichier)[1])
        print('Open file  :  ',fichier)
        
        
        self.newWindow=SEELIGHT(conf=self.conf,name=self.name)
        self.newWindow.setWindowTitle(fichier)
        self.newWindow.newDataReceived(data)
    
    
    def SliderImgFct(self):# open multiimage
        nbImgToOpen=int(self.sliderImage.value())

        self.OpenF(fileOpen=self.openedFiles[nbImgToOpen])
        
    def SaveF (self):
        # save data  in TIFF or Text  files
        
        if self.winOpt.checkBoxTiff.isChecked()==True: 
            fname=QFileDialog.getSaveFileName(self,"Save data as TIFF",self.path)
            self.path=os.path.dirname(str(fname[0]))
            fichier=fname[0]
        
            ext=os.path.splitext(fichier)[1]
            #print(ext)
            print(fichier,' is saved')
            self.conf.setValue(self.name+"/path",self.path)
            time.sleep(0.1)
            self.dataS=np.rot90(self.data,1)
            img_PIL = Image.fromarray(self.dataS)

            img_PIL.save(str(fname[0])+'.TIFF',format='TIFF')
            self.fileName.setText(fname[0]+'.TIFF') 
            
        else :
            fname=QFileDialog.getSaveFileName(self,"Save data as txt",self.path)
            self.path=os.path.dirname(str(fname[0]))
            fichier=fname[0]
            self.dataS=np.rot90(self.data,1)
            ext=os.path.splitext(fichier)[1]
            #print(ext)
            print(fichier,' is saved')
            self.conf.setValue(self.name+"/path",self.path)
            time.sleep(0.1)
            np.savetxt(str(fichier)+'.txt',self.dataS)
            self.fileName.setText(fname[0]+str(ext))

    @pyqtSlot (object) 
    def newDataReceived(self,data):
        
        # Do display and save origin data when new data is  sent to  visu
        
        self.data=data
        self.ImgFrame.animateClick() # change icon data when receive image
        if self.flipButton.isChecked()==1 and self.flipButtonVert.isChecked()==1 :
            self.data=np.flipud(self.data)
            self.data=np.fliplr(self.data)
        elif self.flipButton.isChecked()==1:
            self.data=np.flipud(self.data)
        elif self.flipButtonVert.isChecked()==1 :
            self.data=np.fliplr(self.data)
        else:
            self.data=data
            
        self.data=np.rot90(self.data,self.winPref.rotateValue)
        self.dimy=np.shape(self.data)[1]
        self.dimx=np.shape(self.data)[0]
        self.dataOrgScale=self.data
        self.dataOrg=self.data
        
        self.Display(self.data)
        self.frameName.setText(str(self.frameNumber))
        self.frameNumber=self.frameNumber+1

    
    
    def ScaleImg(self):
        #scale Axis px to um
        if self.winPref.checkBoxAxeScale.isChecked()==1:
            self.scaleAxis="on"
            self.LigneChanged()
        else :
            self.scaleAxis="off"
        self.data=self.dataOrg
        if self.winPref.checkBoxFluence.isChecked()==1: 
            # fluence on 
            self.p1.addItem(self.roiFluence)
            self.fluenceFct()
        else:
            try :
                self.p1.removeItem(self.roiFluence)
            except:
                pass
        self.Display(self.data)
    
    
    def zoomRectAct(self): #zoom fonction Display data on a range difined by a rectangular  roi
        
        if self.plotRectZoomEtat=="Zoom": 
            
            self.p1.addItem(self.plotRectZoom)
            self.plotRectZoom.setSize(size=(2*self.rx,2*self.ry),center=None)
            self.plotRectZoom.setPos([self.xc-1*self.rx,self.yc-1*self.ry])
            
            self.ZoomRectButton.setIcon(QtGui.QIcon(self.icon+"zoom-in.png"))
            self.plotRectZoomEtat="ZoomIn"
            self.ZoomRectButton.setText('Zoom In')
            
        elif self.plotRectZoomEtat=="ZoomIn":
            self.ZoomRectButton.setIcon(QtGui.QIcon(self.icon+"zoom-out.png"))
            self.xZoomMin=(self.plotRectZoom.pos()[0])
            self.yZoomMin=(self.plotRectZoom.pos()[1])
            self.xZoomMax=(self.plotRectZoom.pos()[0])+self.plotRectZoom.size()[0]
            self.yZoomMax=(self.plotRectZoom.pos()[1])+self.plotRectZoom.size()[1]
            self.p1.setXRange(self.xZoomMin,self.xZoomMax)
            
            self.p1.setYRange(self.yZoomMin,self.yZoomMax)
            self.p1.setAspectLocked(False)
            self.p1.removeItem(self.plotRectZoom)
            self.ZoomRectButton.setText('Zoom Out')
            self.plotRectZoomEtat="ZoomOut"
            
        elif self.plotRectZoomEtat=="ZoomOut": 
            self.p1.setYRange(0,self.dimy,update=True)
            self.p1.setXRange(0,self.dimx,update=True)
            #self.p1.setLimits(minXRange=0,maxXRange=self.dimx,minYRange=0,maxYRange=self.dimy,xMin=0,yMin=0,xMax=self.dimx,yMax=self.dimy)
            self.ZoomRectButton.setIcon(QtGui.QIcon(self.icon+"loupe.png"))
            self.ZoomRectButton.setText('Zoom rect')
            self.plotRectZoomEtat="Zoom"
            self.p1.setAspectLocked(True)
            #print(self.p1.viewRange(),self.p1.viewRect())
            
        
        self.Coupe()  #update profile line
        
    def zoomRectupdate(self):
        if self.plotRectZoomEtat=="ZoomOut":
            self.p1.setXRange(self.xZoomMin,self.xZoomMax)
            self.p1.setYRange(self.yZoomMin,self.yZoomMax)
            self.p1.setAspectLocked(True)
        # else:
        #     self.p1.setYRange(0,self.dimy)
        #     self.p1.setXRange(0,self.dimx)
        #     print('ra')
            
    def flipAct (self):
        ##flip the image up/down
        self.data=np.flipud(self.data)
        self.Display(self.data)
    
    def flipVertAct (self):
        #flip the image left/right
        self.data=np.fliplr(self.data)
        self.Display(self.data)
    
    
    def open_widget(self,fene):
        """ open new widget 
        """

        if fene.isWinOpen==False:
            fene.setup
            fene.isWinOpen=True
            
            #fene.Display(self.data)
            fene.show()
        else:
            #fene.activateWindow()
            # fene.raise_()
            fene.showNormal()
    
    
    def dragEnterEvent(self, e):
        e.accept()

        
    def dropEvent(self, e):
        #allow to drop image in visu and open it
        l = []
        for url in e.mimeData().urls():
            l.append(str(url.toLocalFile()))
        e.accept()
        
        if len(l)>1: #open multi file in drag process
                self.openedFiles=l
                self.sliderImage.setMinimum(0)
                self.sliderImage.setMaximum(len(l) - 1)
                self.sliderImage.setValue(0)
                self.sliderImage.setEnabled(True)
                
        
        self.OpenF(fileOpen=l[0])
    
    def checkBoxScaleImage(self):
        if self.checkBoxScale.isChecked()==True:
            self.checkBoxScale.setIcon(QtGui.QIcon(self.icon+"expand.png"))
            self.checkBoxScale.setText('Auto Scale On')
        else :
             self.checkBoxScale.setIcon(QtGui.QIcon(self.icon+"minimize.png"))
             self.checkBoxScale.setText('Auto Scale Off')
    
    def autoSaveColor(self):
        
        if self.checkBoxAutoSave.isChecked()==True:
            self.checkBoxAutoSave.setIcon(QtGui.QIcon(self.icon+"saveAutoOn.png"))
            self.checkBoxAutoSave.setText('Auto Save on')
        else :
             self.checkBoxAutoSave.setIcon(QtGui.QIcon(self.icon+"diskette.png"))
             self.checkBoxAutoSave.setText('Auto Save off')
    
             
    def fluenceFct(self):
        self.sizeFluenceX=self.roiFluence.size()[0]
        self.sizeFluenceY=self.roiFluence.size()[1]
        self.Display(self.data)
    
    
    def ZoomMAX(self):
        self.open_widget(self.winZoomMax)
        self.winZoomMax.SetTITLE('MAX')
        self.maxx=round(self.data.max(),3)
        self.winZoomMax.setZoom(self.maxx)
    
    def closeEvent(self,event):
        self.close()
    def close(self):
        
        # when the window is closed
        if self.encercled=="on":
            if self.winEncercled.isWinOpen==True:
                self.winEncercled.close()
        if self.winCoupe.isWinOpen==True:
            self.winCoupe.close()
        if self.meas=="on":
            if self.winM.isWinOpen==True:
                self.winM.close()
        if self.winOpt.isWinOpen==True:
            self.winOpt.close() 
        if self.winPref.isWinOpen==True:
            self.winPref.close() 
        if self.fft=='on':
            if self.winFFT.isWinOpen==True:
                self.winFFT.close()
            if self.winFFT1D.isWinOpen==True:
                self.winFFT1D.close()
        
       
        
def runVisu(file=None,path=None) :
        
    from pyqtgraph.Qt.QtWidgets import QApplication
    import sys
    import qdarkstyle
    import visu
    
    appli = QApplication(sys.argv)   
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = visu.visual2.SEE2(file=file,path=path)
    e.show()
    appli.exec_() 

   
if __name__ == "__main__":
    
    appli = QApplication(sys.argv) 
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    pathVisu="/Users/juliengautier/Desktop/confTest.ini"
    
    name="testVisu"
    conf=QtCore.QSettings(pathVisu, QtCore.QSettings.Format.IniFormat)
    
    #file='FP__2201_2019_01_17_13_17_59.TIFF'
    #path='/Users/juliengautier/Dropbox (LOA)/Data_Analysis_LOA/Laser X/Manip Janvier 2019/FP/2019-01-16'
    e = SEE(aff='left',roiCross=True)#,color="red")#,conf=conf,name=name)
    e.show()
    appli.exec_() 








# -*- coding: UTF-8
"""
Pilotage des controleurs RSAI via les ddl PilmotTango.dll et openMD.dll
python 3.X pyqtgraph.Qt
system 32 bit (at least python MSC v.1900 32 bit (Intel)) 
@author: Gautier julien loa
Created on Tue Jan 4 10:42:10 2018
modified on Tue Feb 27 15:49:32 2018
"""

#%% Imports
import ctypes
import time

try:
    from pyqtgraph.Qt.QtCore import QSettings
except:
    from PyQt4.QtCore import QSettings


#%% DLL
dll_file = 'DLL/PilMotTango.dll'
#modbus_file='DLL/OpenModbus.dll'
try:
    #PilMot=ctypes.windll.PilMotTango # Chargement de la dll PilMotTango et OpenMD .dll
    PilMot = ctypes.windll.LoadLibrary(dll_file)
    #modbus=ctypes.windll.LoadLibrary(modbus_file)
except AttributeError as s:
    print('########################################################')
    print("Error when loading the dll file : %s" % dll_file)
    print("Error : %s" % s)
    print("PilMot() is then a dummy class.")
    print('########################################################')
    class PilMot():
        """ dummy class """
        def rEtatConnexion(i):
            return i
        def Start(i, s):
            return 0
        def rPositionMot(i, j):
            return 10.
        def wCdeMot(i, j, k, l, m):
            return
        def wPositionMot(i, j, k):
            return
        def rEtatMoteur(i, j):
            return 0
        def Stop():
            return 0

#%% ENTREE
# liste adresse IP des modules
IP    = b"10.0.1.30\0      10.0.1.31\0      10.0.4.30\0      " 
IPs_C = ctypes.create_string_buffer(IP, 48) # permet d avoir la liste comme demander dans la dll


#conf = QSettings(QSettings.IniFormat, QSettings.UserScope, "configMoteur", "configMoteurRSAI")
confRSAI = QSettings('fichiersConfig/configMoteurRSAI.ini', QSettings.IniFormat)




#%% Functions connections RSAI
def startConnexion():
    """ Ouverture d'une connexion avec les racks RSAI """
    print("RSAI initialisation ...")
    argout = 0
    argoutetat = PilMot.rEtatConnexion( ctypes.c_int16(0) ) # numero equipement
    if argoutetat != 3:
        argout = PilMot.Start(ctypes.c_int(3), IPs_C) # nb equipement , liste IP
        if argout == 1 :
            print('RSAI connection : OK RSAI connected @\n', IP)
        else:
            print('RSAI connexion failed')
    return argout

def stopConnexion():
    """ arret des connexions """
    print('RSAI  connexion stopped ')
    PilMot.Stop() # arret de toute les connexion


def testConnection():
    argout = PilMot.rEtatConnexion(ctypes.c_int16(0)) # numero equipement
    if argout == 3:
            print('Test connection OK')
    elif argout == 1:
            print('Already connected at \n', IP)
    else :
        print('Test connexion failed')
    return argout



#%% class MOTORSAI
    
class MOTORRSAI():
    def __init__(self, mot1='',parent=None):
        #super(MOTORNEWPORT, self).__init__()
        self.moteurname=mot1
        self.numEsim=ctypes.c_int16(int(confRSAI.value(self.moteurname+'/numESim')))
        self.numMoteur=ctypes.c_int16(int(confRSAI.value(self.moteurname+'/numMoteur')) )

    def stopMotor(self): # stop le moteur motor
        """ stopMotor(motor): stop le moteur motor """
        regCde = ctypes.c_uint(8) # 8 commande pour arreter le moteur
        PilMot.wCdeMot( self.numEsim , self.numMoteur, regCde, 0, 0)
        print("Stop")


    def move(self, pos, vitesse=10000):
        """
        move(self.moteurname,pos,vitesse): mouvement absolu du moteur (motor) a la position pos avec la vitesse vitesse
        """
        regCde = ctypes.c_uint(2) # commande mvt absolue
        posi   = ctypes.c_int(int(pos))
        vit    = ctypes.c_int( int(vitesse) )
        print(time.strftime("%A %d %B %Y %H:%M:%S"))
        print(self.moteurname, "position before ", self.position(), "(step)")
        PilMot.wCdeMot(self.numEsim , self.numMoteur, regCde, posi, vit)
        print(self.moteurname, "move to", pos, "(step)")

    def rmove(self, posrelatif, vitesse=1000):
        """
        rmove(motor,posrelatif,vitesse): : mouvement absolu du moteur (motor) a la position posrelatif avec la vitesse vitesse
        """
        regCde    = ctypes.c_uint(2) # commande mvt absolue
        posActuel = self.position()
        print(time.strftime("%A %d %B %Y %H:%M:%S"))
        print(self.moteurname,"position before ",posActuel,"(step)")
        pos  = int(posActuel+posrelatif)
        posi = ctypes.c_int(pos)
        vit  = ctypes.c_int(int(vitesse))
        PilMot.wCdeMot(self.numEsim , self.numMoteur, regCde, posi, vit)
        print(self.moteurname, "relative move of", posrelatif, "(step)")

    def setzero(self):
        """
        ## setzero(self.moteurname):Set Zero
        """
        regCde=ctypes.c_int(1024) #  commande pour zero le moteur (2^10)

        a=PilMot.wCdeMot(self.numEsim , self.numMoteur,regCde,ctypes.c_int(0),ctypes.c_int(0))
       
        print (self.moteurname,"zero set",a)


#%% Functions ETAT Moteur

    def etatMotor(self):
        """ Etat du moteur (alimentation on/off)
        a verifier
        """
        a=PilMot.rEtatMoteur(self.numEsim , self.numMoteur)
        if a==8320 or a==41088  or a==128 or a==41104 or a==32869:
            etat=1
        else :
            etat=0
        return etat
        print(self.moteurname,a,etat)

    def position(self):
        """ position (self.moteurname) : donne la postion de motor """
        pos=PilMot.rPositionMot(self.numEsim , self.numMoteur) # lecture position theorique en nb pas
        return pos
 
    
#%% Demarage connexion
        
startConnexion()   

 
#%%
if __name__ == "__main__":
    print("test")
    #startConnexion()





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Feb 13 16:02:40 2019

@author: juliengautier
"""

import qdarkstyle
from PyQt6.QtCore import pyqtSignal as Signal
from PyQt6 import QtCore,QtGui 
from PyQt6.QtWidgets import QApplication,QCheckBox,QVBoxLayout,QHBoxLayout,QPushButton
from  PyQt6.QtWidgets import QWidget,QLabel,QSpinBox,QLineEdit,QMessageBox,QFileDialog
from PyQt6.QtGui import QIcon 
import sys,os
import numpy as np
import socket as _socket
import pathlib
import time

class OPTION(QWidget):
    
    closeEventVar=QtCore.pyqtSignal(bool) 
    emitChangeScale=QtCore.pyqtSignal(bool) 
    emitChangeRot=QtCore.pyqtSignal(bool)
    def __init__(self,conf=None,name='VISU'):
        
        
        
        super().__init__()
        
        p = pathlib.Path(__file__)
        
        if conf==None:
            self.conf=QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else :
            self.conf=conf
        self.name=name
        sepa=os.sep
        self.icon=str(p.parent) + sepa+'icons' +sepa
        self.isWinOpen=False
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
       
        self.setWindowTitle('Options Auto Save & visualisation')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        
        # self.stepX=float(self.conf.value(self.name+"/stepX"))
        # self.stepY=float(self.conf.value(self.name+"/stepY"))
        self.shoot=int(self.conf.value(self.name+"/tirNumber"))
        self.setup()
        self.pathAutoSave=self.conf.value(self.name+"/path")
        self.pathBg=self.conf.value(self.name+"/pathBg")
        self.actionButton()
        self.dataBgExist=False
        self.rotateValue=0

        
        
        
    def setFile(self,file) :
        
        self.nomFichier=file

    def setup(self):
        
        TogOff=self.icon+'Toggle_Off.png'
        TogOn=self.icon+'Toggle_On.png'
        
        
        TogOff=pathlib.Path(TogOff)
        TogOff=pathlib.PurePosixPath(TogOff)
        TogOn=pathlib.Path(TogOn)
        TogOn=pathlib.PurePosixPath(TogOn)
        self.setStyleSheet("QCheckBox::indicator{width: 30px;height: 30px;}""QCheckBox::indicator:unchecked { image : url(%s);}""QCheckBox::indicator:checked { image:  url(%s);}""QCheckBox{font :10pt;}" % (TogOff,TogOn) )
        
        vbox1=QVBoxLayout()
        self.buttonPath=QPushButton('Path : ')
        self.pathBox=QLineEdit(str(self.conf.value(self.name+"/pathAutoSave")))
        self.pathBox.setMaximumHeight(60)
        hbox1=QHBoxLayout()
        hbox1.addWidget(self.buttonPath)
        hbox1.addWidget(self.pathBox)
        
        vbox1.addLayout(hbox1)
        
        hbox2=QHBoxLayout()
        labelName=QLabel('Name : ')
        self.nameBox=QLineEdit(str(self.conf.value(self.name+"/nameFile")))
        self.nameBox.setMaximumHeight(30)
        hbox2.addWidget(labelName)
        hbox2.addWidget(self.nameBox)
       
        vbox1.addLayout(hbox2)
        
        hbox3=QHBoxLayout()
        
         
        self.checkBoxTiff=QCheckBox('Save as TIFF',self)
        self.checkBoxTiff.setChecked(True)
        hbox2.addWidget(self.checkBoxTiff)
        
        self.checkBoxDate=QCheckBox('add date',self)
        self.checkBoxDate.setChecked(False)
        hbox2.addWidget(self.checkBoxDate)
        vbox1.addLayout(hbox3)
        
        hbox4=QHBoxLayout()
        labelTirNumber=QLabel('Next number : ')
        self.checkBoxServer=QCheckBox('Auto',self)
        self.checkBoxServer.setChecked(False)
        hbox4.addWidget(self.checkBoxServer)
        
        self.tirNumberBox=QSpinBox()
        self.tirNumberBox.setMaximum(100000)
        self.tirNumberBox.setValue(self.shoot)
        hbox4.addWidget(labelTirNumber)
        hbox4.addWidget(self.tirNumberBox)
        
        vbox1.addLayout(hbox4)
        
        hbox5=QHBoxLayout()
        self.buttonFileBg=QPushButton('Select Background File : ')
        self.fileBgBox=QLineEdit('bgfile not selected')
        self.fileBgBox.setMaximumHeight(60)
        hbox5=QHBoxLayout()
        hbox5.addWidget(self.buttonFileBg)
        hbox5.addWidget(self.fileBgBox)
        
        vbox1.addLayout(hbox5)
        
        
        hMainLayout=QHBoxLayout()
        hMainLayout.addLayout(vbox1)
        self.setLayout(hMainLayout)
        
    
    def actionButton(self):
        self.buttonPath.clicked.connect(self.PathChanged)
        self.pathBox.textChanged.connect(self.pathTextChanged)
        self.nameBox.textChanged.connect(self.nameFileChanged)
        self.tirNumberBox.valueChanged.connect(self.TirNumberChange)
        self.buttonFileBg.clicked.connect(self.selectBg)
        #self.fileBgBox.textChanged.connect(self.bgTextChanged)
        self.checkBoxServer.stateChanged.connect(self.checkBoxServerChange)

    def pathTextChanged(self):
        self.pathAutoSave=self.pathBox.text()
        self.conf.setValue(self.name+"/pathAutoSave",str(self.pathAutoSave))
        
    def PathChanged(self) :
       
        self.pathAutoSave=str(QFileDialog.getExistingDirectory(self,"Select Directory",self.pathAutoSave))
        self.pathBox.setText(self.pathAutoSave)
        self.conf.setValue(self.name+"/pathAutoSave",str(self.pathAutoSave))
        
    def nameFileChanged(self):
        self.fileName=self.nameBox.text()
        self.conf.setValue(self.name+"/nameFile",str(self.fileName))
            
    def TirNumberChange(self):
        self.tirNumber=self.tirNumberBox.value()
        self.conf.setValue(self.name+"/tirNumber",self.tirNumber)
        self.conf.sync()
        
    def setTirNumber(self,tirNumber) :
        self.tirNumber=tirNumber
        self.tirNumberBox.setValue(self.tirNumber)
        
    
    # def rotateChange(self):
    #     self.rotateValue=int(self.rotate.value())
        
    
    
    def selectBg(self):
        
        fname=QFileDialog.getOpenFileName(self,"Select a background file",self.pathBg,"Images (*.txt *.spe *.TIFF *.sif);;Text File(*.txt);;Ropper File (*.SPE);;Andor File(*.sif);; TIFF file(*.TIFF)")
        fichier=fname[0]
        ext=os.path.splitext(fichier)[1]
        self.fileBgBox.setText(fichier)
        
        self.conf.setValue(self.name+"/pathBg",os.path.dirname(fichier))
        
        self.dataBgExist=True
        
        if ext=='.txt': # text file
            self.dataBg=np.loadtxt(str(fichier))
        elif ext=='.spe' or ext=='.SPE': # SPE file
            from winspec import SpeFile
            dataSPE=SpeFile(fichier)
            data1=dataSPE.data[0]#.transpose() # first frame
            self.dataBg=data1#np.flipud(data1)
        elif ext=='.TIFF':# tiff File
            from PIL import Image
            dat=Image.open(fichier)
            self.dataBg=np.array(dat)
        elif ext=='.sif': 
            from andor import SifFile
            sifop=SifFile()
            im=sifop.openA(fichier)
            self.dataBg=np.rot90(im,3)
            
        else :
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Icon.Critical)
            msg.setText("Wrong file format !")
            msg.setInformativeText("The format of the file must be : .SPE  .TIFF .sif or .txt ")
            msg.setWindowTitle("Warning ...")
            msg.setWindowFlags(QtCore.Qt.WindowType.WindowStaysOnTopHint)
            msg.exec_()
            self.dataBgExist=False
    
        
    def checkBoxServerChange(self):
        
        if self. checkBoxServer.isChecked()==True:
            # print('start number of shot Client')
            self.threadClient=THREADCLIENT(self)
            self.threadClient.start()
            self.threadClient.newShotnumber.connect(self.receiveNewNumber)
        else:
            # print('close client')
            self.threadClient.stopClientThread()
    
    def receiveNewNumber (self,nbShot):
        self.tirNumberBox.setValue(nbShot)
        #print(nbShot)
    
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen=False   
        self.closeEventVar.emit(True)
        
class THREADCLIENT(QtCore.QThread):
    
    '''Second thread for controling one acquisition independtly
    '''
    newShotnumber=Signal(int)#QtCore.Signal(int) # signal to send 
    
    def __init__(self, parent):
        
        super(THREADCLIENT,self).__init__(parent)
        self.parent=parent
        self.conf=self.parent.conf
        self.name=self.parent.name
        self.serverHost = str(self.conf.value(self.name+"/server"))
        self.serverPort = str(self.conf.value(self.name+"/serverPort"))
        self.clientSocket= _socket.socket(_socket.AF_INET, _socket.SOCK_STREAM) 
        
        
    def run(self):
        try :
            self.serverHost = str(self.conf.value(self.name+"/server"))
            self.serverPort = str(self.conf.value(self.name+"/serverPort"))
            print(self.serverHost,self.serverPort)
            self.clientSocket.connect((self.serverHost,int(self.serverPort)))
            
            self.ClientIsConnected = True
            print('client connected to server',self.serverHost)
        except:
            self.isConnected = False
            print('Do you start the server?')
            self.ClientIsConnected = False
            self.parent.checkBoxServer.setChecked(False)
            
        
       
        while self.ClientIsConnected == True:
            cmd='numberShoot?'
            self.clientSocket.send(cmd.encode())
            try:
                receiv=self.clientSocket.recv(64500)
                nbshot=int(receiv.decode())
            except:
                self.parent.checkBoxServer.setChecked(False)
            if int(self.parent.tirNumberBox.value()) is not nbshot: # sent signal only when different
                   self.newShotnumber.emit(nbshot)
            time.sleep(0.1)
     
    def stopClientThread(self):
            self.ClientIsConnected=False
            self.clientSocket.close()
     
if __name__ == "__main__":
    appli = QApplication(sys.argv) 
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = OPTION() 
    e.show()
    appli.exec_() 


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Apr 12 21:39:01 2021

@author: juliengautier
"""


from PyQt6.QtWidgets import QApplication,QVBoxLayout,QWidget
from PyQt6.QtWidgets import QLabel
from PyQt6.QtGui import QIcon

from PyQt6.QtCore import Qt
from pyqtgraph.Qt.QtGui import QIcon
import visu
import pathlib,os,sys
import qdarkstyle
__version__=visu.__version__
__author__=visu.__author__


class ABOUT(QWidget):
    
    def __init__(self,file=None,path=None,**kwds):
        
        super().__init__()
        self.version=__version__
        p = pathlib.Path(__file__)
        sepa=os.sep
        self.title="About Visualization"
        self.icon=str(p.parent) + sepa+'icons' +sepa
        self.isWinOpen=False
        self.setWindowTitle(self.title)
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet_pyqt5())
        self.setup()
        
    def setup(self):
        hMainLayout=QVBoxLayout()
        img=self.icon+'LOA.png'
        
        text1=QLabel('Visualization  Version :  '+self.version)
        text1.setStyleSheet("font:22pt")
        text1.setAlignment(Qt.AlignmentFlag.AlignCenter)
        
        text2=QLabel("<a href=\"https://loa.ensta-paris.fr/fr/accueil/\">Laboratoire d'optique Appliquée</a>")
        text2.setAlignment(Qt.AlignCenter)
        
        text2.setOpenExternalLinks(True)
        text2.setTextFormat(Qt.RichText)
        
        text3=QLabel(__author__)
        text3.setAlignment(Qt.AlignCenter)
        text31=QLabel("<a href=\"https://github.com/julienGautier77/visu/wiki/How--to-use-Visu\">visit on GitHub</a>")
        text31.setOpenExternalLinks(True)
        text31.setTextFormat(Qt.RichText)
        
        
        text4=QLabel()
        text4.setStyleSheet("QLabel:!pressed{border-image: url(%s);background-color: transparent ;border-color: green;}""QLabel:pressed{image: url(%s);background-color: gray ;border-color: gray}"% (img,img))           
        text4.setMaximumWidth(60)
        text4.setMaximumHeight(60)
        text4.setAlignment(Qt.AlignCenter)
        
        hMainLayout.addWidget(text1)
        hMainLayout.addWidget(text2)
        hMainLayout.addWidget(text3)
        hMainLayout.addWidget(text31)
        hMainLayout.addWidget(text4)
        self.setLayout(hMainLayout)
        
   
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen=False
        
        event.accept()
        
if __name__ == "__main__":
    appli = QApplication(sys.argv) 
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = ABOUT()  
    e.show()
    appli.exec_()     
    


# -*- coding: utf-8 -*-
"""
Created on Tue Aug 13 15:27:31 2019
windows for see max ,mean ,sum in a bigger windows
@author: Salle-Jaune
"""



import qdarkstyle # pip install qdakstyle https://github.com/ColinDuquesnoy/QDarkStyleSheet  sur conda
from PyQt6.QtWidgets import QApplication,QHBoxLayout,QWidget,QLabel
from PyQt6.QtGui import QIcon
import sys,time

import pathlib,os

class ZOOM(QWidget):
    
    def __init__(self,symbol=True,title='Zoom'):
        super().__init__()
        p = pathlib.Path(__file__)
        sepa=os.sep
        self.title=title
        self.icon=str(p.parent) + sepa+'icons' +sepa
        self.isWinOpen=False
        self.symbol=symbol
        self.setup()
        self.setWindowTitle(self.title)
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    
    def setup(self):
        hLayout2=QHBoxLayout()
        self.label=QLabel('?')
        self.label.setMinimumHeight(100)
        self.label.setMinimumWidth(400)
        self.label.setMaximumHeight(200)
        self.label.setMaximumWidth(500)
        self.label.setStyleSheet('font :bold  60pt;color: white')
        hLayout2.addWidget(self.label)
        self.setLayout(hLayout2)
        
    def setZoom(self,dat):
        if self.title=='Sum':
            self.label.setText('%.3e' % dat)
        else :
            self.label.setText('%.0f' % dat)
        
        
    def SetTITLE(self,title):
        self.title=title
        self.setWindowTitle(title)
        
        
        
        
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen=False
        time.sleep(0.1)
        event.accept()
    
    
if __name__ == "__main__":
    appli = QApplication(sys.argv) 
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = ZOOM()  
    e.show()
    appli.exec_()     


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar 18 10:50:06 2019
Data vizualisation
@author: juliengautier
"""
__version__='2022.09'
__author__='julien Gautier'


from visu import visual
from visu import visualLight

from visu.visualLight import SEELIGHT


from visu.visual import SEE
from visu.visual import runVisu
from visu import andor
from visu import WinCut
from visu import winMeas
from visu import winspec
from visu import winSuppE
from visu import winFFT
from visu import winZoom
from visu import winMath
from visu import winPointing
from visu import winHist
# try:
#     from visu import Win3D
# except :
#     print('')
# #from visu import moteurRSAI as RSAI



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Dec 19 11:43:05 2018
@author: juliengautier

"""

from PyQt6.QtWidgets import QApplication,QVBoxLayout,QHBoxLayout,QWidget,QGridLayout
from PyQt6.QtWidgets import QCheckBox,QLabel,QSizePolicy,QSpinBox,QSizePolicy
from PyQt6 import QtCore,QtGui 
from PyQt6.QtCore import Qt
from PyQt6.QtGui import QShortcut
from PyQt6.QtGui import QIcon
import sys,time
import pyqtgraph as pg # pyqtgraph biblio permettent l'affichage 
import numpy as np
import qdarkstyle # pip install qdakstyle https://github.com/ColinDuquesnoy/QDarkStyleSheet  sur conda
import pylab,os
from scipy.ndimage.filters import gaussian_filter # pour la reduction du bruit
from scipy.interpolate import splrep, sproot # pour calcul fwhm et fit 
import pathlib

class WINENCERCLED(QWidget):
    
    def __init__(self,parent=None,conf=None,name='VISU'):
        
        super().__init__()
        self.name=name
        self.parent=parent
        p = pathlib.Path(__file__)
        if conf==None:
            self.conf=QtCore.QSettings(str(p.parent / 'confVisu.ini'), QtCore.QSettings.Format.IniFormat)
        else :
            self.conf=conf
        
        sepa=os.sep
        self.icon=str(p.parent) + sepa+'icons' +sepa

        self.isWinOpen=False
        self.setWindowTitle('Encercled')
        self.setWindowIcon(QIcon(self.icon+'LOA.png'))
        self.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
        self.left=100
        self.top=30
        self.width=800
        self.height=800
        self.setGeometry(self.left,self.top,self.width,self.height)
        self.dimx=1200
        self.dimy=900
        self.bloqq=1
        self.xec=int(self.conf.value(self.name+"/xec"))
        self.yec=int(self.conf.value(self.name+"/yec"))
        self.r1x=int(self.conf.value(self.name+"/r1x"))
        self.r1y=int(self.conf.value(self.name+"/r1y"))
        self.r2=int(self.conf.value(self.name+"/r2x"))
        self.r2=int(self.conf.value(self.name+"/r2y"))
        
        self.kE=0 # variable pour la courbe E fct du nb shoot
        
        self.Xec=[]
        self.Yec=[]
        self.fwhmX=100
        self.fwhmY=100
        self.setWindowIcon(QIcon('./icons/LOA.png'))
        self.E = []
        
        #self.E=np.array([2,3,5])
        self.Xec=[]
        self.Yec=[]
        
        # Create x and y indices
        x = np.arange(0,self.dimx)
        y = np.arange(0,self.dimy)
        y,x = np.meshgrid(y, x)
    
        self.data=(40*np.random.rand(self.dimx,self.dimy)).round()
        
        self.setup()
        self.ActionButton()
        self.Display(self.data)
        
    def setup(self):
        
        TogOff=self.icon+'Toggle_Off.png'
        TogOn=self.icon+'Toggle_On.png'
        
        
        TogOff=pathlib.Path(TogOff)
        TogOff=pathlib.PurePosixPath(TogOff)
        TogOn=pathlib.Path(TogOn)
        TogOn=pathlib.PurePosixPath(TogOn)
        self.setStyleSheet("QCheckBox::indicator{width: 30px;height: 30px;}""QCheckBox::indicator:unchecked { image : url(%s);}""QCheckBox::indicator:checked { image:  url(%s);}""QCheckBox{font :10pt;}" % (TogOff,TogOn) )
        
        vbox1=QVBoxLayout()
        hbox=QHBoxLayout()
        self.checkBoxAuto=QCheckBox('Auto',self)
        self.checkBoxAuto.setChecked(True)
        hbox.addWidget(self.checkBoxAuto)
        self.bckButton=QCheckBox('Soustract Bg',self)
        hbox.addWidget(self.bckButton)
        vbox1.addLayout(hbox)
        hbox0=QHBoxLayout()
        self.energieRes=QLabel('?')
        self.energieRes.setMaximumHeight(30)
        self.energieRes.setMaximumWidth(140)
        self.energieRes.setStyleSheet("color:blue;font:20pt")
        self.lEnergie=QLabel('s(E1)/s(E2) :')
        self.lEnergie.setStyleSheet("color:blue;font:18pt")
        self.lEnergie.setMaximumWidth(160)
        hbox0.addWidget(self.lEnergie)
        hbox0.addWidget(self.energieRes)
        #vbox1.addStretch(1)
        vbox1.addLayout(hbox0)
        
        
        LabelR1x=QLabel("fwhm X/0.85")
        LabelR1x.setStyleSheet("color:red;font:14pt")
        self.r1xBox=QSpinBox()
        #self.r1Box.setMaximumWidth(60)
        self.r1xBox.setMaximum(2000)
        LabelR1y=QLabel('fwhm Y/0.85')
        LabelR1y.setStyleSheet("color:green;font:14pt")
        self.r1yBox=QSpinBox()
        self.r1yBox.setMaximum(2000)
        #self.r2Box.setMaximumWidth(60)
        LabelR2=QLabel('R2')
        LabelR2.setStyleSheet("color:yellow;font:14pt")
        self.r2Box=QSpinBox()
        self.r2Box.setMaximum(2000)
        
        LabelE1=QLabel("E1 Sum ")
        LabelE1.setStyleSheet("color:red;font:14pt")
        self.LabelE1Sum=QLabel("? ")
        self.LabelE1Sum.setStyleSheet("color:red;font:14pt")
        LabelE1M=QLabel("E1 mean ")
        LabelE1M.setStyleSheet("color:red;font:14pt")
        self.LabelE1Mean=QLabel("? ")
        self.LabelE1Mean.setStyleSheet("color:red;font:14pt")
        
        
        LabelE2=QLabel("E2 Sum ")
        LabelE2.setStyleSheet("color:yellow;font:14pt")
        self.LabelE2Sum=QLabel("? ")
        self.LabelE2Sum.setStyleSheet("color:yellow;font:14pt")
        LabelE2M=QLabel("E2 mean ")
        LabelE2M.setStyleSheet("color:yellow ;font:14pt")
        self.LabelE2Mean=QLabel("? ")
        self.LabelE2Mean.setStyleSheet("color:yellow;font:14pt")
        
        grid_layout1 = QGridLayout()
        grid_layout1.addWidget(LabelR1x, 0, 0)
        grid_layout1.addWidget(self.r1xBox, 0, 1)
        grid_layout1.addWidget(LabelR1y, 1, 0)
        grid_layout1.addWidget(self.r1yBox, 1,1)
        grid_layout1.addWidget(LabelR2, 2, 0)
        grid_layout1.addWidget(self.r2Box, 2,1)
        grid_layout1.addWidget(LabelE1,3,0)
        grid_layout1.addWidget(self.LabelE1Sum,3,1)
        grid_layout1.addWidget(LabelE1M,4,0)
        grid_layout1.addWidget(self.LabelE1Mean,4,1)
        
        grid_layout1.addWidget(LabelE2,5,0)
        grid_layout1.addWidget(self.LabelE2Sum,5,1)
        grid_layout1.addWidget(LabelE2M,6,0)
        grid_layout1.addWidget(self.LabelE2Mean,6,1)
        
        
        vbox1.addLayout(grid_layout1)
        
        vbox1.addStretch(1)
        
        self.winImage = pg.GraphicsLayoutWidget()
        self.winImage.setContentsMargins(0,0,0,0)
        self.winImage.setAspectLocked(True)
        self.winImage.setSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.MinimumExpanding)
        #self.winImage.ci.setContentsMargins(0,0,0,0)
        
        vbox2=QVBoxLayout()
        hbox2=QHBoxLayout()
        
        
        hbox2.addWidget(self.winImage)
        vbox2.addLayout(hbox2)
        vbox2.setContentsMargins(0,0,0,0)
        
        self.p1=self.winImage.addPlot()
        
        self.imh=pg.ImageItem()
        self.p1.addItem(self.imh)
        
        self.p1.setMouseEnabled(x=False,y=False)
        self.p1.setContentsMargins(0,0,0,0)
        
        self.p1.setAspectLocked(True,ratio=1)
        self.p1.showAxis('right',show=False)
        self.p1.showAxis('top',show=False)
        self.p1.showAxis('left',show=False)
        self.p1.showAxis('bottom',show=False)
        self.vLine = pg.InfiniteLine(angle=90, movable=False,pen='w')
        self.hLine = pg.InfiniteLine(angle=0, movable=False,pen='w')
        self.p1.addItem(self.vLine)
        self.p1.addItem(self.hLine)
        
        self.vLine.setPos(self.xec)
        self.hLine.setPos(self.yec)
        
        self.roi1=pg.CircleROI([self.xec,self.yec],[2*self.r1x,2*self.r1y],pen='r',movable=False)
        self.roi1.setPos([self.xec-(self.r1x),self.yec-(self.r1y)])
        self.p1.addItem(self.roi1)
       
        self.roi2=pg.CircleROI([self.xec,self.yec],[2*self.r2,2*self.r2],pen='y',movable=False)
        self.roi2.setPos([self.xec-(self.r2),self.yec-(self.r2)])
        self.p1.addItem(self.roi2)
        
        #histogramme
        self.hist = pg.HistogramLUTItem() 
        self.hist.setImageItem(self.imh)
        self.hist.autoHistogramRange()
        self.hist.gradient.loadPreset('flame')
        
        self.curve2=pg.PlotCurveItem()
        self.curve3=pg.PlotCurveItem()
        # text pour afficher fwhm sur p1
        self.textX = pg.TextItem(angle=-90) 
        self.textY = pg.TextItem()
        self.p1.addItem(self.curve2)
        self.p1.addItem(self.curve3)
        self.p1.addItem(self.textX)
        self.p1.addItem(self.textY)
        
        
        self.ROIRect=pg.RectROI([self.xec,self.yec],[4*self.r1x,4*self.r1y],pen='m',)
        
        hLayout1=QHBoxLayout()
        
        hLayout1.addLayout(vbox2)
        hLayout1.addLayout(vbox1)
        
        
        hLayout1.setContentsMargins(1,1,1,1)
#        hLayout1.setSpacing(1)
#        hLayout1.setStretch(10,1)
        
        vMainLayout=QVBoxLayout()
        vMainLayout.addLayout(hLayout1)
        
        
        
        
        
        hMainLayout=QHBoxLayout()
        hMainLayout.addLayout(vMainLayout)
        self.setLayout(hMainLayout)
        self.setContentsMargins(1,1,1,1)
        
        
        
    def ActionButton(self):   
        # Blocage de la souris
        self.roi1.sigRegionChangeFinished.connect(self.energSouris) # signal si changement à la souris de la taille des cercles
        self.roi2.sigRegionChangeFinished.connect(self.energSouris)
#
        self.r1xBox.editingFinished.connect(self.Rayon) #w.r1Box.returnPressed.connect(Rayon)# rayon change a la main
        self.r1yBox.editingFinished.connect(self.Rayon)
        self.r2Box.editingFinished.connect(self.Rayon)
       
        self.shortcutb=QtGui.QShortcut(QtGui.QKeySequence("Ctrl+b"),self)
        self.shortcutb.activated.connect(self.bloquer)
        self.shortcutd=QtGui.QShortcut(QtGui.QKeySequence("Ctrl+d"),self)
        self.shortcutd.activated.connect(self.debloquer)
         
        self.shortcutPu=QShortcut(QtGui.QKeySequence("+"),self)
        self.shortcutPu.activated.connect(self.paletteup)
        self.shortcutPu.setContext(Qt.ShortcutContext(3))
        #3: The shortcut is active when its parent widget, or any of its children has focus. default O The shortcut is active when its parent widget has focus.
        self.shortcutPd=QtGui.QShortcut(QtGui.QKeySequence("-"),self)
        self.shortcutPd.activated.connect(self.palettedown)
        self.shortcutPd.setContext(Qt.ShortcutContext(3))
        
        self.checkBoxAuto.stateChanged.connect(lambda:self.Display(self.data))
        # mvt de la souris
        self.proxy=pg.SignalProxy(self.p1.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)
        self.p1.scene().sigMouseClicked.connect(self.mouseClick)
        
        self.vb=self.p1.vb
        
        self.checkBoxAuto.stateChanged.connect(self.AutoE)
        self.bckButton.stateChanged.connect(self.Back)
        self.ROIRect.sigRegionChangeFinished.connect(self.roiBackChanged)
        
        
        if self.parent is not None:
            self.parent.signalEng.connect(self.Display)
        
    def mouseClick(self):
        # bloque ou debloque la souris si click su le graph
        
        if self.bloqq==1:
            self.debloquer()
        else :
            self.bloquer()  
            
    def bloquer(self): # bloque la croix 
        self.bloqq=1
        self.conf.setValue(self.name+"/xec",int(self.xec)) # save cross postion in ini file
        self.conf.setValue(self.name+"/yec",int(self.yec))
        self.CalculE()
        
    def debloquer(self): # deblaoque la croix : elle bouge avec la souris
        self.bloqq=0
    
    # mvt de la souris
    
    def mouseMoved(self,evt):
        ## pour que la cross suive le mvt de la souris
        if self.checkBoxAuto.isChecked()==False:

            if self.bloqq==0: # souris non bloquer
                pos = evt[0]  ## using signal proxy turns original arguments into a tuple
                if self.p1.sceneBoundingRect().contains(pos):
                    mousePoint = self.vb.mapSceneToView(pos)
                    self.xec =int (mousePoint.x())
                    self.yec= int(mousePoint.y())
                    if ((self.xec>0 and self.xec<self.data.shape[0]) and (self.yec>0 and self.yec<self.data.shape[1]) ):
                            self.vLine.setPos(self.xec)
                            self.hLine.setPos(self.yec) # la croix ne bouge que dans le graph  
                            
                            self.roi1.setPos([self.xec-(self.r1x),self.yec-(self.r1y)])
                            self.roi2.setPos([self.xec-(self.r2),self.yec-(self.r2)])
                            
    
    def AutoE(self):
        if self.checkBoxAuto.isChecked()==False:
            self.energSouris()
            self.roi1.setSize([2*self.r1x,2*self.r1y])
            self.roi2.setSize([2*self.r2,2*self.r2])
            
            
    def energSouris(self): # changement des rayons à la souris 
        if self.checkBoxAuto.isChecked()==False:
            s1=self.roi1.size()
            s2=self.roi2.size()
            self.r1xBox.setValue((int(s1[0]/2)))
            self.r1yBox.setValue((int(s1[1]/2)))
            self.r2Box.setValue((int(s2[0]/2)))
            self.r1x=int(s1[0]/2)
            self.r1y=int(s1[1]/2)
            self.r2=int(s2[0]/2)
            if self.bloqq==1:
                self.CalculE()
     
        
    def Rayon(self): 
        """changement rayon dans les box
        """
        if self.checkBoxAuto.isChecked()==False:
            if self.r1xBox.hasFocus() :
                self.r1x=(float(self.r1xBox.text()))
                self.roi1.setSize([2*self.r1x,2*self.r1y])
            if self.r1yBox.hasFocus() :
                self.r1y=(float(self.r1yBox.text()))
                self.roi1.setSize([2*self.r1x,2*self.r1y])
            
            if self.r2Box.hasFocus():
                self.r2=(float(self.r2Box.text()))
                self.roi2.setSize([2*self.r2,2*self.r2])
            
            self.roi1.setPos([self.xec-(self.r1x),self.yec-(self.r1y)])
            self.roi2.setPos([self.xec-(self.r2),self.yec-(self.r2)])
            if self.bloqq==1:
                self.CalculE()
    
    
    def CalculE(self):
        
        if self.fwhmX==None or self.fwhmY==None:
            self.fwhmX=100
            self.fwhmY=100
        
        if self.checkBoxAuto.isChecked()==True:
            self.r1x=self.fwhmX/0.849
            self.r1y=self.fwhmY/0.849
            self.r1xBox.setValue(int(self.r1x))
            self.r1yBox.setValue(int(self.r1y))
            nbG=2 # ré= 2 fois r1 pour le grand cercle
            self.roi2.setSize([2*nbG*self.r1x,2*nbG*self.r1y])
            self.roi2.setPos([self.xec-nbG*self.r1x,self.yec-nbG*self.r1y])
            self.r2Box.setValue(int(2*nbG*self.r1x))
            self.roi1.setSize([2*self.r1x,2*self.r1y])
            self.roi1.setPos([self.xec-self.r1x,self.yec-self.r1y])
            
        else:
            self.r1x=self.r1x
            self.r1y=self.r1y
           
        E1=self.roi1.getArrayRegion(self.data,self.imh).sum()
        E2=self.roi2.getArrayRegion(self.data,self.imh).sum()
        self.rap=100*E1/E2
        self.energieRes.setText('%.2f %%' % self.rap)
        # self.E=np.append(self.E,self.rap)
        # #self.E.append(self.rap)
        # Emean=np.mean(self.E)
        # self.meanAff.setText('%.2f' % Emean)
        # EPV=np.std(self.E)
        # self.PVAff.setText('%.2f' % EPV)
        # self.hLineMeanE.setPos(Emean)
        self.textX.setText('fwhm='+str(self.fwhmY))
        self.textY.setText('fwhm='+str(self.fwhmX),color='w')
        self.LabelE1Sum.setText('%.2f' %E1)
        self.LabelE1Mean.setText('%.2f' % (self.roi1.getArrayRegion(self.data,self.imh).mean()))
        self.LabelE2Sum.setText('%.2f' %E2)
        self.LabelE2Mean.setText('%.2f' % (self.roi2.getArrayRegion(self.data,self.imh).mean()))
        
    def Display(self,data):
        self.dataOrg=data
        self.data=data
        
        self.dimx=self.data.shape[0]
        self.dimy=self.data.shape[1]
        self.p1.setXRange(0,self.dimx)
        self.p1.setYRange(0,self.dimy)
        self.p1.setAspectLocked(True,ratio=1)
        self.imh.setImage(data.astype(float),autoLevels=True,autoDownsample=True)
        self.CalculCentroid()
        self.Coupe()
        self.Back()
        # self.CalculE() 
        
    #     self.Xec.append(self.xec)
    #     self.Yec.append(self.yec)
        
    #     Xmean=np.mean(self.Xec)
    #     self.meanXAff.setText('%.2f' % Xmean)
    #     XPV=np.std(self.Xec)
    #     self.stdXAff.setText('%.2f' % XPV)
    #     self.hLineMeanX.setPos(Xmean)
    #     Ymean=np.mean(self.Yec)
    #     self.meanYAff.setText('%.2f' % Ymean)
    #     YPV=np.std(self.Yec)
    #     self.stdYAff.setText('%.2f' % YPV)
    #     self.hLineMeanY.setPos(Ymean)
    #     self.plotGraph()
       
        
    
    # def plotGraph(self):
        
        
        
    #     self.p2.setData(self.E)#,pen='b',symbol='t',symboleSize=2,clear=True,symbolPen='b',symbolBrush='b',name="rapport")
    #     #self.p2.addItem(self.hLineMeanE, ignoreBounds=True)
    #     self.p3.setData(self.Xec)#,pen='r',symbol='t',symboleSize=2,clear=True,symbolPen='r',symbolBrush='r',name="x")
    #     #self.p3.addItem(self.hLineMeanX, ignoreBounds=True)
    #     self.p4.setData(self.Yec)#,pen='g',symbol='t',symboleSize=2,clear=True,symbolPen='g',symbolBrush='g',name="y")
    #     #self.p4.addItem(self.hLineMeanY, ignoreBounds=True)
    
    
    
    def CalculCentroid(self):
        
        if self.checkBoxAuto.isChecked()==True:
            dataF=gaussian_filter(self.data,5)
            (self.xec,self.yec)=pylab.unravel_index(dataF.argmax(),self.data.shape) #prend le max 
            self.vLine.setPos(self.xec)
            self.hLine.setPos(self.yec)       
            self.roi1.setPos([self.xec-(self.r1x),self.yec-(self.r1y)])
            self.roi2.setPos([self.xec-(self.r2),self.yec-(self.r2)])
        
        
    def fwhm(self,x, y, order=3):
        
        """
            Determine full-with-half-maximum of a peaked set of points, x and y.
            Assumes that there is only one peak present in the datasset.  The function
            uses a spline interpolation of order k.
        """
        y=gaussian_filter(y,5) # filtre pour reduire le bruit
        half_max = np.amax(y)/2
        try:
            s = splrep(x, y - half_max,k=order) # Find the B-spline representation of 1-D curve.
            roots = sproot(s) # Given the knots (>=8) and coefficients of a cubic B-spline return the roots of the spline.
        except:
           roots=0
           
        if len(roots) > 2:
            pass
            #print( "The dataset appears to have multiple peaks, and ","thus the FWHM can't be determined.")
        elif len(roots) < 2:
            pass
           # print( "No proper peaks were found in the data set; likely ","the dataset is flat (e.g. all zeros).")
        else:
            return np.around(abs(roots[1] - roots[0]),decimals=2)
         
    def Coupe(self):
        
        xxx=np.arange(0,int(self.dimx),1)#
        yyy=np.arange(0,int(self.dimy),1)#
        coupeX=self.data[int(self.xec),:]
        coupeXMax=np.max(coupeX)
        
        if coupeXMax==0: # evite la div par zero
            coupeXMax=1
            
        coupeXnorm=(self.data.shape[0]/10)*(coupeX/coupeXMax) # normalise les coupes pour pas prendre tout l ecran
        self.curve2.setData(30+coupeXnorm,yyy,clear=True)
        
        coupeY=self.data[:,int(self.yec)]
        coupeYMax=np.max(coupeY)
        if coupeYMax==0:
            coupeYMax=1
            
        coupeYnorm=(self.data.shape[1]/10)*(coupeY/coupeYMax)
        self.curve3.setData(xxx,20+coupeYnorm,clear=True)
        
        ### affichage de fwhm sur les coupes X et Y que si le max est >20 counts
        xCXmax=np.amax(coupeXnorm) # max
        if xCXmax>20:
            self.fwhmY=self.fwhm(yyy, coupeXnorm, order=3)
            
            
            yCXmax=yyy[coupeXnorm.argmax()]
            self.textX.setPos(xCXmax-3,yCXmax+3)
            
        yCYmax=np.amax(coupeYnorm) # max
        if yCYmax>20:
            self.fwhmX=self.fwhm(xxx, coupeYnorm, order=3)
            xCYmax=xxx[coupeYnorm.argmax()]            
            
            self.textY.setPos(xCYmax-3,yCYmax-3) 

    def paletteup(self):
        levels=self.imh.getLevels()
        if levels[0]==None:
            xmax =self.data.max()
            xmin=self.data.min()
        else :
            xmax=levels[1]
            xmin=levels[0]
            
        self.imh.setLevels([xmin, xmax+(xmax- xmin) / 10])
        #hist.setImageItem(imh,clear=True)
        self.hist.setHistogramRange(xmin,xmax)

    def palettedown(self):
        levels=self.imh.getLevels()
        if levels[0]==None:
            xmax=self.data.max()
            xmin=self.data.min()
        else :
            xmax=levels[1]
            xmin=levels[0]
            
        self.imh.setLevels([xmin, xmax- (xmax- xmin) / 10])
        #hist.setImageItem(imh,clear=True)
        self.hist.setHistogramRange(xmin,xmax)
        
    # def Reset(self):
    #     self.E = []
    #     self.Xec=[]
    #     self.Yec=[]
        
    
        
    def roiBackChanged(self):
        bg=self.ROIRect.getArrayRegion(self.dataOrg,self.imh).mean()
        print(bg)
        self.data=self.dataOrg-bg
        self.CalculE()
    
    def Back(self):
        if self.bckButton.isChecked()==True:
            self.p1.addItem(self.ROIRect)
            bg=self.ROIRect.getArrayRegion(self.dataOrg,self.imh).mean()
            self.data=self.dataOrg-bg
            self.CalculE()
        else:
            self.p1.removeItem(self.ROIRect)
            
            self.data=self.dataOrg
            
            self.CalculE()
            
            
            
    def closeEvent(self, event):
        """ when closing the window
        """
        self.isWinOpen=False
        self.E=[]
        self.Xec=[]
        self.Yec=[]
        time.sleep(0.1)
        event.accept()
     
        
  
        
if __name__ == "__main__":
    appli = QApplication(sys.argv) 
    appli.setStyleSheet(qdarkstyle.load_stylesheet(qt_api='pyqt6'))
    e = WINENCERCLED(name='VISU')  
    e.show()
    appli.exec_()         